@@ -67,13 +67,9 @@ typedef struct
   int            tableID;
   int            comptype;       // compression type
   int            complevel;      // compression level
-  short          timave;
   bool           lmissval;
   double         missval;
   char          *name;
-  char          *stdname;
-  char          *longname;
-  char          *units;
 
   /* meta-data for specification of tiles (currently only GRIB-API: */
   subtype_t     *tiles;
@@ -99,7 +95,6 @@ void paramInitEntry(unsigned varID, int param)
   vartable[varID].prec           = 0;
   vartable[varID].tsteptype      = TSTEP_INSTANT;
   varScanKeysInit(&vartable[varID].scanKeys);
-  vartable[varID].timave         = 0;
   vartable[varID].gridID         = CDI_UNDEFID;
   vartable[varID].zaxistype      = 0;
   vartable[varID].ltype1         = 0;
@@ -113,16 +108,12 @@ void paramInitEntry(unsigned varID, int param)
   vartable[varID].instID         = CDI_UNDEFID;
   vartable[varID].modelID        = CDI_UNDEFID;
   vartable[varID].tableID        = CDI_UNDEFID;
-  vartable[varID].keys.nalloc    = MAX_KEYS;
-  vartable[varID].keys.nelems    = 0;
+  cdiInitKeys(&vartable[varID].keys);
   vartable[varID].comptype       = CDI_COMPRESS_NONE;
   vartable[varID].complevel      = 1;
   vartable[varID].lmissval       = false;
   vartable[varID].missval        = 0;
   vartable[varID].name           = NULL;
-  vartable[varID].stdname        = NULL;
-  vartable[varID].longname       = NULL;
-  vartable[varID].units          = NULL;
   vartable[varID].tiles          = NULL;
 }
 
@@ -175,9 +166,6 @@ void varFree(void)
         }
 
       if ( vartable[varID].name )     Free(vartable[varID].name);
-      if ( vartable[varID].stdname )  Free(vartable[varID].stdname);
-      if ( vartable[varID].longname ) Free(vartable[varID].longname);
-      if ( vartable[varID].units )    Free(vartable[varID].units);
       if ( vartable[varID].tiles )    subtypeDestroyPtr(vartable[varID].tiles);
 
       cdi_keys_t *keysp = &(vartable[varID].keys);
@@ -399,11 +387,10 @@ int varInsertTileSubtype(vartable_t *vptr, const var_tile_t *tiles)
 
 void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
 		  int level1, int level2, int level_sf, int level_unit, int prec,
-		  int *pvarID, int *plevelID, int tsteptype, int numavg, int ltype1, int ltype2,
-		  const char *name, const char *stdname, const char *longname, const char *units,
-                  const VarScanKeys *scanKeys, const var_tile_t *tiles, int *tile_index)
+		  int *pvarID, int *plevelID, int tsteptype, int ltype1, int ltype2,
+		  const char *name, const VarScanKeys *scanKeys, const var_tile_t *tiles, int *tile_index)
 {
-  unsigned varID = (cdiSplitLtype105 != 1 || zaxistype != ZAXIS_HEIGHT) ?
+  unsigned varID = (CDI_Split_Ltype105 != 1 || zaxistype != ZAXIS_HEIGHT) ?
     varGetEntry(param, gridID, zaxistype, ltype1, tsteptype, name, scanKeys, tiles) : (unsigned) CDI_UNDEFID;
 
   if ( varID == (unsigned) CDI_UNDEFID )
@@ -420,12 +407,7 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
       vartable[varID].tsteptype  = tsteptype;
       if (scanKeys) vartable[varID].scanKeys = *scanKeys;
 
-      if ( numavg ) vartable[varID].timave = 1;
-
       if ( name && name[0] )         vartable[varID].name     = strdup(name);
-      if ( stdname && stdname[0] )   vartable[varID].stdname  = strdup(stdname);
-      if ( longname && longname[0] ) vartable[varID].longname = strdup(longname);
-      if ( units && units[0] )       vartable[varID].units    = strdup(units);
     }
   else
     {
@@ -487,9 +469,7 @@ int cmpLevelTable(const void* s1, const void* s2)
   int cmp = 0;
   const leveltable_t *x = (const leveltable_t*) s1;
   const leveltable_t *y = (const leveltable_t*) s2;
-  /*
-  printf("%g %g  %d %d\n", x->leve11, y->level1, x, y);
-  */
+  // printf("%g %g  %d %d\n", x->leve11, y->level1, x, y);
   if      ( x->level1 < y->level1 ) cmp = -1;
   else if ( x->level1 > y->level1 ) cmp =  1;
 
@@ -502,9 +482,7 @@ int cmpLevelTableInv(const void* s1, const void* s2)
   int cmp = 0;
   const leveltable_t *x = (const leveltable_t*) s1;
   const leveltable_t *y = (const leveltable_t*) s2;
-  /*
-  printf("%g %g  %d %d\n", x->leve11, y->level1, x, y);
-  */
+  // printf("%g %g  %d %d\n", x->leve11, y->level1, x, y);
   if      ( x->level1 < y->level1 ) cmp =  1;
   else if ( x->level1 > y->level1 ) cmp = -1;
 
@@ -514,8 +492,9 @@ int cmpLevelTableInv(const void* s1, const void* s2)
 
 void varCopyKeys(int vlistID, int varID)
 {
-  cdi_keys_t *keysp = &(vartable[varID].keys);
-  cdiCopyVarKeys(keysp, vlistID, varID);
+  vlist_t *vlistptr = vlist_to_pointer(vlistID);
+  cdiInitKeys(&vlistptr->vars[varID].keys);
+  cdiCopyVarKeys(&vartable[varID].keys, &vlistptr->vars[varID].keys);
 }
 
 
@@ -573,16 +552,15 @@ void cdi_generate_vars(stream_t *streamptr)
       const int param      = vartable[varid].param;
       const int ltype1     = vartable[varid].ltype1;
       const int ltype2     = vartable[varid].ltype2;
-      int zaxistype  = vartable[varid].zaxistype;
+      int zaxistype        = vartable[varid].zaxistype;
       if ( ltype1 == 0 && zaxistype == ZAXIS_GENERIC && cdiDefaultLeveltype != -1 )
 	zaxistype = cdiDefaultLeveltype;
       const int lbounds    = vartable[varid].lbounds;
       const int prec       = vartable[varid].prec;
-      int instID     = vartable[varid].instID;
-      int modelID    = vartable[varid].modelID;
-      int tableID    = vartable[varid].tableID;
+      int instID           = vartable[varid].instID;
+      int modelID          = vartable[varid].modelID;
+      int tableID          = vartable[varid].tableID;
       const int tsteptype  = vartable[varid].tsteptype;
-      const int timave     = vartable[varid].timave;
       const int comptype   = vartable[varid].comptype;
 
       double level_sf = 1;
@@ -624,11 +602,12 @@ void cdi_generate_vars(stream_t *streamptr)
 
       double *dlevels = (double *) Malloc(nlevels*sizeof(double));
 
+      /*
       if ( lbounds && zaxistype != ZAXIS_HYBRID && zaxistype != ZAXIS_HYBRID_HALF )
 	for ( unsigned levelID = 0; levelID < nlevels; levelID++ )
-	  dlevels[levelID] = (level_sf*levelTable[levelID].level1 +
-	                      level_sf*levelTable[levelID].level2)/2;
+	  dlevels[levelID] = (level_sf*levelTable[levelID].level1 + level_sf*levelTable[levelID].level2) / 2.0;
       else
+      */
 	for ( unsigned levelID = 0; levelID < nlevels; levelID++ )
 	  dlevels[levelID] = level_sf*levelTable[levelID].level1;
 
@@ -667,11 +646,12 @@ void cdi_generate_vars(stream_t *streamptr)
 
           if ( lsort )
             {
+              /*
               if ( lbounds && zaxistype != ZAXIS_HYBRID && zaxistype != ZAXIS_HYBRID_HALF )
                 for ( unsigned levelID = 0; levelID < nlevels; levelID++ )
-                  dlevels[levelID] = (level_sf*levelTable[levelID].level1 +
-                                      level_sf*levelTable[levelID].level2)/2.;
+                  dlevels[levelID] = (level_sf*levelTable[levelID].level1 + level_sf*levelTable[levelID].level2) / 2.0;
               else
+              */
                 for ( unsigned levelID = 0; levelID < nlevels; levelID++ )
                   dlevels[levelID] = level_sf*levelTable[levelID].level1;
             }
@@ -694,40 +674,36 @@ void cdi_generate_vars(stream_t *streamptr)
       int zaxisID = varDefZaxis(vlistID, zaxistype, (int)nlevels, dlevels, cvals, 0, lbounds, dlevels1, dlevels2,
                                 (int)Vctsize, Vct, NULL, NULL, unitptr, 0, 0, ltype1, ltype2);
 
-      if ( CDI_cmor_mode && nlevels == 1 && zaxistype != ZAXIS_HYBRID ) zaxisDefScalar(zaxisID);
+      if ( CDI_CMOR_Mode && nlevels == 1 && zaxistype != ZAXIS_HYBRID ) zaxisDefScalar(zaxisID);
 
       if ( zaxisInqType(zaxisID) == ZAXIS_REFERENCE )
         {
-          if ( numberOfVerticalLevels > 0 ) zaxisDefNlevRef(zaxisID, numberOfVerticalLevels);
-          if ( numberOfVerticalGrid > 0 ) zaxisDefNumber(zaxisID, numberOfVerticalGrid);
-          if ( !cdiUUIDIsNull(uuidVGrid) ) zaxisDefUUID(zaxisID, uuidVGrid);
+          if ( numberOfVerticalLevels > 0 ) cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NLEV, numberOfVerticalLevels);
+          if ( numberOfVerticalGrid > 0 ) cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NUMBEROFVGRIDUSED, numberOfVerticalGrid);
+          if ( !cdiUUIDIsNull(uuidVGrid) ) cdiDefKeyBytes(zaxisID, CDI_GLOBAL, CDI_KEY_UUID, uuidVGrid, CDI_UUID_SIZE);
         }
 
       if ( lbounds ) Free(dlevels1);
       if ( lbounds ) Free(dlevels2);
       Free(dlevels);
 
-      /* define new subtype for tile set */
+      // define new subtype for tile set
       int tilesetID = CDI_UNDEFID;
       if ( vartable[varid].tiles ) tilesetID = vlistDefTileSubtype(vlistID, vartable[varid].tiles);
 
-      /* generate new variable */
+      // generate new variable
       int varID = stream_new_var(streamptr, gridID, zaxisID, tilesetID);
       varID = vlistDefVarTiles(vlistID, gridID, zaxisID, TIME_VARYING, tilesetID);
 
       vlistDefVarTsteptype(vlistID, varID, tsteptype);
       vlistDefVarParam(vlistID, varID, param);
       vlistDefVarDatatype(vlistID, varID, prec);
-      vlistDefVarTimave(vlistID, varID, timave);
       vlistDefVarCompType(vlistID, varID, comptype);
 
       varCopyKeys(vlistID, varID);
 
       if ( vartable[varid].lmissval ) vlistDefVarMissval(vlistID, varID, vartable[varid].missval);
-      if ( vartable[varid].name )     vlistDefVarName(vlistID, varID, vartable[varid].name);
-      if ( vartable[varid].stdname )  vlistDefVarStdname(vlistID, varID, vartable[varid].stdname);
-      if ( vartable[varid].longname ) vlistDefVarLongname(vlistID, varID, vartable[varid].longname);
-      if ( vartable[varid].units )    vlistDefVarUnits(vlistID, varID, vartable[varid].units);
+      if ( vartable[varid].name ) cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, vartable[varid].name);
 
       vlist_t *vlistptr = vlist_to_pointer(vlistID);
       for ( int i = 0; i < vartable[varid].opt_grib_nentries; i++ )
@@ -743,29 +719,29 @@ void cdi_generate_vars(stream_t *streamptr)
 	      strdupx(vartable[varid].opt_grib_kvpair[i].keyword);
           vlistptr->vars[varID].opt_grib_kvpair[i].update = true;
         }
-      /* note: if the key is not defined, we do not throw an error! */
+      // note: if the key is not defined, we do not throw an error!
 
-      if ( cdiDefaultTableID != CDI_UNDEFID )
+      if ( CDI_Default_TableID != CDI_UNDEFID )
 	{
 	  int pdis, pcat, pnum;
 	  cdiDecodeParam(param, &pnum, &pcat, &pdis);
           char name[CDI_MAX_NAME]; name[0] = 0;
           char longname[CDI_MAX_NAME]; longname[0] = 0;
           char units[CDI_MAX_NAME]; units[0] = 0;
-          tableInqEntry(cdiDefaultTableID, pnum, -1, name, longname, units);
+          tableInqEntry(CDI_Default_TableID, pnum, -1, name, longname, units);
 	  if ( name[0] )
 	    {
 	      if ( tableID != CDI_UNDEFID )
 		{
-		  vlistDefVarName(vlistID, varID, name);
-                  if ( longname[0] ) vlistDefVarLongname(vlistID, varID, longname);
-                  if ( units[0] ) vlistDefVarUnits(vlistID, varID, units);
+		  cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, name);
+                  if ( longname[0] ) cdiDefKeyString(vlistID, varID, CDI_KEY_LONGNAME, longname);
+                  if ( units[0] ) cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, units);
                 }
 	      else
-		tableID = cdiDefaultTableID;
+		tableID = CDI_Default_TableID;
 	    }
-	  if ( cdiDefaultModelID != CDI_UNDEFID ) modelID = cdiDefaultModelID;
-	  if ( cdiDefaultInstID  != CDI_UNDEFID )  instID = cdiDefaultInstID;
+	  if ( CDI_Default_ModelID != CDI_UNDEFID ) modelID = CDI_Default_ModelID;
+	  if ( CDI_Default_InstID  != CDI_UNDEFID )  instID = CDI_Default_InstID;
 	}
 
       if ( instID  != CDI_UNDEFID ) vlistDefVarInstitut(vlistID, varID, instID);
@@ -826,8 +802,11 @@ bool zaxisCompare(int zaxisID, int zaxistype, int nlevels, bool lbounds, const d
 {
   bool differ = true;
 
-  bool ltype1_is_equal = (ltype1 == zaxisInqLtype(zaxisID));
-  bool ltype2_is_equal = (ltype2 == zaxisInqLtype2(zaxisID));
+  int ltype1_0 = 0, ltype2_0 = -1;
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype1_0);
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFSECONDFIXEDSURFACE, &ltype2_0);
+  bool ltype1_is_equal = (ltype1 == ltype1_0);
+  bool ltype2_is_equal = (ltype2 == ltype2_0);
 
   if ( ltype1_is_equal && ltype2_is_equal && (zaxistype == zaxisInqType(zaxisID) || zaxistype == ZAXIS_GENERIC) )
     {
@@ -850,14 +829,16 @@ bool zaxisCompare(int zaxisID, int zaxistype, int nlevels, bool lbounds, const d
 	    {
               if ( longname && longname[0] )
                 {
-                  char zlongname[CDI_MAX_NAME]; zlongname[0] = 0;
-                  zaxisInqLongname(zaxisID, zlongname);
+                  char zlongname[CDI_MAX_NAME];
+                  int length = CDI_MAX_NAME;
+                  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, zlongname, &length);
                   if ( zlongname[0] && (strcmp(longname, zlongname) != 0) ) differ = true;
                 }
               if ( units && units[0] )
                 {
-                  char zunits[CDI_MAX_NAME]; zunits[0] = 0;
-                  zaxisInqUnits(zaxisID, zunits);
+                  char zunits[CDI_MAX_NAME];
+                  int length = CDI_MAX_NAME;
+                  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, zunits, &length);
                   if ( zunits[0] && (strcmp(units, zunits) != 0) ) differ = true;
                 }
             }
@@ -968,13 +949,12 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, const double *levels, c
 	  if ( (zaxistype == ZAXIS_HYBRID || zaxistype == ZAXIS_HYBRID_HALF) && vctsize > 0 )
             zaxisDefVct(zaxisID, vctsize, vct);
 
-	  if ( name && name[0] ) zaxisDefName(zaxisID, name);
-	  if ( longname && longname[0] ) zaxisDefLongname(zaxisID, longname);
-	  if ( units && units[0] ) zaxisDefUnits(zaxisID, units);
+	  if ( name && name[0] ) cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, name);
+	  if ( longname && longname[0] ) cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, longname);
+	  if ( units && units[0] ) cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units);
 	  zaxisDefDatatype(zaxisID, prec);
-	  zaxisDefLtype(zaxisID, ltype1);
-          //if ( ltype1 != ltype2 && ltype2 != -1 ) zaxisDefLtype2(zaxisID, ltype2);
-          if ( ltype2 != -1 ) zaxisDefLtype2(zaxisID, ltype2);
+          cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, ltype1);
+          if ( ltype2 != -1 ) cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFSECONDFIXEDSURFACE, ltype2);
 	}
 
       vlistptr->zaxisIDs[nzaxis] = zaxisID;
@@ -1055,6 +1035,14 @@ void varDefKeyBytes(int varID, int key, const unsigned char *bytes, int length)
 }
 
 
+void varDefKeyString(int varID, int key, const char *string)
+{
+  int length = strlen(string) + 1;
+  cdi_keys_t *keysp = &(vartable[varID].keys);
+  cdiDefVarKeyBytes(keysp, key, (const unsigned char *) string, length);
+}
+
+
 #ifdef HAVE_LIBGRIB_API
 /* Resizes and initializes opt_grib_kvpair data structure. */
 static
