@@ -1,20 +1,11 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
+#include <cstdlib>
 #include <cdi.h>
 
 #include "cdo_options.h"
@@ -83,7 +74,7 @@ MakeWhiteNoise(int n_xres, int n_yres, unsigned char *pNoise)
   for (int j = 0; j < n_yres; j++)
     for (int i = 0; i < n_xres; i++)
       {
-        int r = rand();
+        int r = std::rand();
         r = ((r & 0xff) + ((r & 0xff00) >> 8)) & 0xff;
         pNoise[j * n_xres + i] = (unsigned char) r;
       }
@@ -403,7 +394,7 @@ lic1(const char *obasename, int num, int n_xres, int n_yres, float *pVectr)
           if (mag[i] < vmin) vmin = mag[i];
           if (mag[i] > vmax) vmax = mag[i];
         }
-      cdoPrint("ts=%d minval=%g  maxval=%g", num + 1, vmin, vmax);
+      cdo_print("ts=%d minval=%g  maxval=%g", num + 1, vmin, vmax);
     }
 
   NormalizVectrs(n_xres, n_yres, pVectr);
@@ -418,7 +409,7 @@ lic1(const char *obasename, int num, int n_xres, int n_yres, float *pVectr)
 #ifdef HAVE_LIBPNG
   WriteImage2PNG(n_xres, n_yres, pImage, mag, filename);
 #else
-  cdoWarning("PNG support not compiled in!");
+  cdo_warning("PNG support not compiled in!");
 #endif
 
   free(p_LUT0);
@@ -431,43 +422,41 @@ lic1(const char *obasename, int num, int n_xres, int n_yres, float *pVectr)
 void *
 Lic(void *process)
 {
-  int nrecs;
-  int varID, levelID;
   int nlev = 0;
   size_t nmiss;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  // int LIC  = cdoOperatorAdd("lic",  0, 0, nullptr);
+  // int LIC  = cdo_operator_add("lic",  0, 0, nullptr);
   // clang-format on
 
-  // int operatorID = cdoOperatorID();
+  // int operatorID = cdo_operator_id();
 
-  if (operatorArgc() < 1 || operatorArgc() > 2) operatorCheckArgc(1);
-  const char *cpt_file = cdoOperatorArgv(0).c_str();
-  int nstart = (operatorArgc() == 2) ? atoi(cdoOperatorArgv(1).c_str()) : 0;
+  if (cdo_operator_argc() < 1 || cdo_operator_argc() > 2) operator_check_argc(1);
+  const char *cpt_file = cdo_operator_argv(0).c_str();
+  int nstart = (cdo_operator_argc() == 2) ? std::stoi(cdo_operator_argv(1)) : 0;
   auto cpt_fp = fopen(cpt_file, "r");
-  if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
+  if (cpt_fp == nullptr) cdo_abort("Open failed on color palette table %s", cpt_file);
 
-  int status = cptRead(cpt_fp, &cpt);
-  if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
+  int status = cpt_read(cpt_fp, &cpt);
+  if (status != 0) cdo_abort("Error during read of color palette table %s", cpt_file);
 
-  const auto obasename = cdoGetStreamName(1);
-  const auto streamID1 = cdoOpenRead(0);
+  const auto obasename = cdo_get_stream_name(1);
+  const auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
 
   // int taxisID1 = vlistInqTaxis(vlistID1);
 
   /* find variables */
   const auto nvars = vlistNvars(vlistID1);
-  if (nvars != 2) cdoAbort("Need 2 input variable, found %d\n", nvars);
+  if (nvars != 2) cdo_abort("Need 2 input variable, found %d\n", nvars);
   int varID1 = 0;
   int varID2 = 1;
 
   const auto ngrids = vlistNgrids(vlistID1);
-  if (ngrids != 1) cdoAbort("Need 1 grid, found %d\n", ngrids);
+  if (ngrids != 1) cdo_abort("Need 1 grid, found %d\n", ngrids);
 
   const auto gridID = vlistInqVarGrid(vlistID1, varID1);
   // int zaxisID = vlistInqVarZaxis(vlistID1, varID1);
@@ -490,31 +479,33 @@ Lic(void *process)
     }
 
   const auto gridtype = gridInqType(gridID);
-  if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN) cdoAbort("Unsupported grid!\n");
+  if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN) cdo_abort("Unsupported grid!\n");
 
   const auto nx = gridInqXsize(gridID);
   const auto ny = gridInqYsize(gridID);
 
-  Varray<double> xvals(nx);
-  Varray<double> yvals(ny);
+  Varray<double> xvals(nx), yvals(ny);
   gridInqYvals(gridID, yvals.data());
   bool invertlat = (yvals[0] < yvals[ny - 1]);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+      if (nrecs == 0) break;
       /*
-      taxisCopyTimestep(taxisID2, taxisID1);
-      cdoDefTimestep(streamID2, tsID);
+      cdo_taxis_copy_timestep(taxisID2, taxisID1);
+      cdo_def_timestep(streamID2, tsID);
       */
       for (int recID = 0; recID < nrecs; recID++)
         {
-          cdoInqRecord(streamID1, &varID, &levelID);
+          int varID, levelID;
+          cdo_inq_record(streamID1, &varID, &levelID);
 
           if (varID == varID1 || varID == varID2)
             {
-              cdoReadRecord(streamID1, array1.data(), &nmiss);
-              if (nmiss) cdoAbort("Missing values unsupported!");
+              cdo_read_record(streamID1, array1.data(), &nmiss);
+              if (nmiss) cdo_abort("Missing values unsupported!");
 
               gridsize = gridInqSize(gridID);
 
@@ -540,9 +531,9 @@ Lic(void *process)
       tsID++;
     }
 
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID1);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
