@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -32,22 +22,20 @@
 
 #include <cdi.h>
 
-#include "array.h"
+#include "varray.h"
 #include "cdo_options.h"
 #include "process_int.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
-#include "gridreference.h"
 #include "color.h"
 #include "printinfo.h"
 #include "interpol.h"
 #include "cdo_zaxis.h"
 
-
 static int
 check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
 {
-  int ncorner_new = ncorner;
+  auto ncorner_new = ncorner;
   int k;
 
   for (k = ncorner - 1; k > 0; --k)
@@ -59,29 +47,100 @@ check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
 }
 
 static void
-make_cyclic(double *array1, double *array2, int nlon, int nlat)
+make_cyclic(double *array1, double *array2, long nlon, long nlat)
 {
-  int ij1, ij2;
-
-  for (int j = 0; j < nlat; ++j)
-    for (int i = 0; i < nlon; ++i)
+  for (long j = 0; j < nlat; ++j)
+    for (long i = 0; i < nlon; ++i)
       {
-        ij1 = j * nlon + i;
-        ij2 = j * (nlon + 1) + i;
+        const long ij1 = j * nlon + i;
+        const long ij2 = j * (nlon + 1) + i;
         array2[ij2] = array1[ij1];
       }
 
-  for (int j = 0; j < nlat; ++j)
+  for (long j = 0; j < nlat; ++j)
     {
-      ij2 = j * (nlon + 1);
+      const long ij2 = j * (nlon + 1);
       array2[ij2 + nlon] = array2[ij2];
     }
 }
 
 static void
-output_vrml(int nlon, int nlat, int ngp, const Varray<double> &array, double missval, CPT *cpt)
+output_zon(double levmin, double levmax, const double *cell_corner_lat)
 {
-  const auto minmax = varrayMinMaxMV(ngp, array, missval);
+  double xlev[4], xlat[4];
+  auto latmin = cell_corner_lat[0];
+  auto latmax = cell_corner_lat[0];
+  for (int ic = 1; ic < 4; ic++) latmin = std::min(latmin, cell_corner_lat[ic]);
+  for (int ic = 1; ic < 4; ic++) latmax = std::max(latmax, cell_corner_lat[ic]);
+  xlev[0] = levmin;
+  xlev[1] = levmax;
+  xlev[2] = levmax;
+  xlev[3] = levmin;
+  xlat[0] = latmin;
+  xlat[1] = latmin;
+  xlat[2] = latmax;
+  xlat[3] = latmax;
+  for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlat[ic], xlev[ic]);
+  fprintf(stdout, "   %g  %g\n", xlat[0], xlev[0]);
+}
+
+static void
+output_mer(double levmin, double levmax, const double *cell_corner_lon)
+{
+  double xlev[4], xlon[4];
+  auto lonmin = cell_corner_lon[0];
+  auto lonmax = cell_corner_lon[0];
+  for (int ic = 1; ic < 4; ic++) lonmin = std::min(lonmin, cell_corner_lon[ic]);
+  for (int ic = 1; ic < 4; ic++) lonmax = std::max(lonmax, cell_corner_lon[ic]);
+  xlev[0] = levmin;
+  xlev[1] = levmin;
+  xlev[2] = levmax;
+  xlev[3] = levmax;
+  xlon[0] = lonmin;
+  xlon[1] = lonmax;
+  xlon[2] = lonmax;
+  xlon[3] = lonmin;
+  for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlon[ic], xlev[ic]);
+  fprintf(stdout, "   %g  %g\n", xlon[0], xlev[0]);
+}
+
+static void
+output_boundscpt(double value, double missval, const CPT &cpt)
+{
+  int r = 0, g = 0, b = 0, n;
+
+  if (!DBL_IS_EQUAL(value, missval))
+    {
+      for (n = 0; n < cpt.ncolors; n++)
+        if (value > cpt.lut[n].z_low && value <= cpt.lut[n].z_high) break;
+
+      if (n == cpt.ncolors)
+        {
+          r = cpt.bfn[0].rgb[0];
+          g = cpt.bfn[0].rgb[1];
+          b = cpt.bfn[0].rgb[2];
+        }
+      else
+        {
+          r = cpt.lut[n].rgb_high[0];
+          g = cpt.lut[n].rgb_high[1];
+          b = cpt.lut[n].rgb_high[2];
+        }
+    }
+  else
+    {
+      r = cpt.bfn[2].rgb[0];
+      g = cpt.bfn[2].rgb[1];
+      b = cpt.bfn[2].rgb[2];
+    }
+
+  fprintf(stdout, " -G%d/%d/%d", r, g, b);
+}
+
+static void
+output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double missval, const CPT &cpt)
+{
+  const auto mm = varray_min_max_mv(ngp, array, missval);
 
   double dx = 10. / nlon;
 
@@ -116,44 +175,43 @@ output_vrml(int nlon, int nlat, int ngp, const Varray<double> &array, double mis
   printf("      geometry ElevationGrid {\n");
   printf("        colorPerVertex true\n");
   printf("        solid false\n");
-  printf("        xDimension %d\n", nlon);
-  printf("        zDimension %d\n", nlat);
+  printf("        xDimension %ld\n", nlon);
+  printf("        zDimension %ld\n", nlat);
   printf("        xSpacing %g\n", dx);
   printf("        zSpacing %g\n", dx);
   printf("        color Color {\n");
   printf("          color [\n");
-  for (int j = nlat - 1; j >= 0; --j)
-    for (int i = 0; i < nlon; ++i)
+  for (long j = nlat - 1; j >= 0; --j)
+    for (long i = 0; i < nlon; ++i)
       {
         int r = 0, g = 0, b = 0;
-        double val = array[j * nlon + i];
+        const auto val = array[j * nlon + i];
 
         if (!DBL_IS_EQUAL(val, missval))
           {
             int n;
-            for (n = 0; n < cpt->ncolors; n++)
-              if (val > cpt->lut[n].z_low && val <= cpt->lut[n].z_high) break;
+            for (n = 0; n < cpt.ncolors; n++)
+              if (val > cpt.lut[n].z_low && val <= cpt.lut[n].z_high) break;
 
-            if (n == cpt->ncolors)
+            if (n == cpt.ncolors)
               {
-                r = cpt->bfn[0].rgb[0];
-                g = cpt->bfn[0].rgb[1];
-                b = cpt->bfn[0].rgb[2];
+                r = cpt.bfn[0].rgb[0];
+                g = cpt.bfn[0].rgb[1];
+                b = cpt.bfn[0].rgb[2];
               }
             else
               {
-                //  r = cpt->lut[n].rgb_high[0];  g = cpt->lut[n].rgb_high[1];
-                //  b = cpt->lut[n].rgb_high[2];
-                r = intlin(val, cpt->lut[n].rgb_low[0], cpt->lut[n].z_low, cpt->lut[n].rgb_high[0], cpt->lut[n].z_high);
-                g = intlin(val, cpt->lut[n].rgb_low[1], cpt->lut[n].z_low, cpt->lut[n].rgb_high[1], cpt->lut[n].z_high);
-                b = intlin(val, cpt->lut[n].rgb_low[2], cpt->lut[n].z_low, cpt->lut[n].rgb_high[2], cpt->lut[n].z_high);
+                //  r = cpt.lut[n].rgb_high[0];  g = cpt.lut[n].rgb_high[1];  b = cpt.lut[n].rgb_high[2];
+                r = intlin(val, cpt.lut[n].rgb_low[0], cpt.lut[n].z_low, cpt.lut[n].rgb_high[0], cpt.lut[n].z_high);
+                g = intlin(val, cpt.lut[n].rgb_low[1], cpt.lut[n].z_low, cpt.lut[n].rgb_high[1], cpt.lut[n].z_high);
+                b = intlin(val, cpt.lut[n].rgb_low[2], cpt.lut[n].z_low, cpt.lut[n].rgb_high[2], cpt.lut[n].z_high);
               }
           }
         else
           {
-            r = cpt->bfn[2].rgb[0];
-            g = cpt->bfn[2].rgb[1];
-            b = cpt->bfn[2].rgb[2];
+            r = cpt.bfn[2].rgb[0];
+            g = cpt.bfn[2].rgb[1];
+            b = cpt.bfn[2].rgb[2];
           }
         printf(" %.3g %.3g %.3g,\n", r / 255., g / 255., b / 255.);
       }
@@ -161,8 +219,8 @@ output_vrml(int nlon, int nlat, int ngp, const Varray<double> &array, double mis
   printf("        }\n");
   printf("        height [\n");
 
-  for (int j = nlat - 1; j >= 0; --j)
-    for (int i = 0; i < nlon; ++i) printf("%g,\n", array[j * nlon + i]);
+  for (long j = nlat - 1; j >= 0; --j)
+    for (long i = 0; i < nlon; ++i) printf("%g,\n", array[j * nlon + i]);
 
   printf("        ]\n");
   printf("      }\n");
@@ -170,17 +228,39 @@ output_vrml(int nlon, int nlat, int ngp, const Varray<double> &array, double mis
   printf("  ]\n");
   printf("  translation -5 0 %g\n", -5. * nlat / nlon);
   printf("  rotation 0.0 0.0 0.0 0.0\n");
-  printf("  scale 1.0 %g 1.0\n", 0.5 / (minmax.second - minmax.first));
+  printf("  scale 1.0 %g 1.0\n", 0.5 / (mm.max - mm.min));
   printf("}\n");
 }
 
+static void
+output_vector(long nlon, long nlat, int ninc, const Varray<double> &lon, const Varray<double> &lat, const Varray<double> &uf,
+              const Varray<double> &vf)
+{
+  for (long j = 0; j < nlat; j += ninc)
+    for (long i = 0; i < nlon; i += ninc)
+      {
+        const auto idx = j * nlon + i;
+
+        // compute length of velocity vector
+        const auto auv = std::sqrt(uf[idx] * uf[idx] + vf[idx] * vf[idx]);
+
+        auto alpha = std::atan2(vf[idx], uf[idx]);
+        alpha = 90. - alpha * RAD2DEG;
+
+        if (alpha < 0) alpha += 360;
+        if (alpha > 360) alpha -= 360;
+
+        if (std::fabs(auv) > 0) fprintf(stdout, " %g  %g  %g  %g\n", lon[idx], lat[idx], alpha, auv);
+      }
+
+  fprintf(stdout, "#\n");
+}
+
 void *
 Outputgmt(void *process)
 {
   int varID0;
   size_t gridsize2 = 0;
-  int nrecs;
-  int levelID;
   size_t nmiss;
   int ninc = 1;
   bool lzon = false, lmer = false, lhov = false;
@@ -190,50 +270,49 @@ Outputgmt(void *process)
   Varray<double> grid_corner_lat, grid_corner_lon;
   std::vector<int> grid_mask;
   CPT cpt;
-  char units[CDI_MAX_NAME];
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
-  const auto OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
-  const auto OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
-  const auto OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
-  const auto OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
-  const auto OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
-  const auto OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
-  const auto OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
+  const auto OUTPUTCENTER    = cdo_operator_add("gmtxyz",          0, 0, nullptr);
+  const auto OUTPUTCENTER2   = cdo_operator_add("outputcenter2",   0, 0, nullptr);
+  const auto OUTPUTCENTERCPT = cdo_operator_add("outputcentercpt", 0, 0, nullptr);
+  const auto OUTPUTBOUNDS    = cdo_operator_add("gmtcells",        0, 0, nullptr);
+  const auto OUTPUTBOUNDSCPT = cdo_operator_add("outputboundscpt", 0, 0, nullptr);
+  const auto OUTPUTVECTOR    = cdo_operator_add("outputvector",    0, 0, nullptr);
+  const auto OUTPUTTRI       = cdo_operator_add("outputtri",       0, 0, nullptr);
+  const auto OUTPUTVRML      = cdo_operator_add("outputvrml",      0, 0, nullptr);
   // clang-format on
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
 
   if (operatorID == OUTPUTVECTOR)
     {
-      operatorInputArg("increment");
-      operatorCheckArgc(1);
-      ninc = parameter2int(cdoOperatorArgv(0));
-      if (ninc < 1) cdoAbort("Increment must be greater than 0!");
+      operator_input_arg("increment");
+      operator_check_argc(1);
+      ninc = parameter_to_int(cdo_operator_argv(0));
+      if (ninc < 1) cdo_abort("Increment must be greater than 0!");
     }
 
   if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) luse_grid_corner = true;
 
   if (operatorID == OUTPUTCENTERCPT || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTVRML)
     {
-      operatorCheckArgc(1);
-      auto cpt_file = cdoOperatorArgv(0).c_str();
+      operator_check_argc(1);
+      auto cpt_file = cdo_operator_argv(0).c_str();
 
       auto cpt_fp = fopen(cpt_file, "r");
-      if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
+      if (cpt_fp == nullptr) cdo_abort("Open failed on color palette table %s", cpt_file);
 
-      const auto status = cptRead(cpt_fp, &cpt);
-      if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
+      const auto status = cpt_read(cpt_fp, &cpt);
+      if (status != 0) cdo_abort("Error during read of color palette table %s", cpt_file);
 
-      if (Options::cdoVerbose) cptWrite(stderr, cpt);
+      if (Options::cdoVerbose) cpt_write(stderr, cpt);
     }
 
-  const auto streamID = cdoOpenRead(0);
+  const auto streamID = cdo_open_read(0);
 
-  const auto vlistID = cdoStreamInqVlist(streamID);
+  const auto vlistID = cdo_stream_inq_vlist(streamID);
   const auto taxisID = vlistInqTaxis(vlistID);
 
   int varID = 0;
@@ -243,38 +322,15 @@ Outputgmt(void *process)
   const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
   const auto missval = vlistInqVarMissval(vlistID, varID);
 
+  gridID = generate_full_cell_grid(gridID);
+
+  if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
+
   const auto gridsize = gridInqSize(gridID);
 
-  int gridtype = gridInqType(gridID);
-  if (gridtype == GRID_GME || gridtype == GRID_GAUSSIAN_REDUCED)
-    {
-      gridID = gridToUnstructured(gridID, 1);
-      gridtype = gridInqType(gridID);
-    }
-
-  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
-    {
-      gridID = gridToCurvilinear(gridID, 1);
-      gridtype = gridInqType(gridID);
-    }
-
-  if (gridtype == GRID_UNSTRUCTURED)
-    {
-      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-        {
-          if (gridInqNumber(gridID) > 0)
-            {
-              gridID = referenceToGrid(gridID);
-              if (gridID == -1) cdoAbort("Reference to source grid not found!");
-            }
-        }
-    }
-
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
-
-  int nlon = gridInqXsize(gridID);
-  int nlat = gridInqYsize(gridID);
-  int nlev = zaxisInqSize(zaxisID);
+  long nlon = gridInqXsize(gridID);
+  long nlat = gridInqYsize(gridID);
+  long nlev = zaxisInqSize(zaxisID);
 
   if (gridInqMaskGME(gridID, nullptr))
     {
@@ -293,19 +349,19 @@ Outputgmt(void *process)
       nlat = 1;
     }
 
-  if (Options::cdoVerbose && lhov) cdoPrint("Process hovmoeller data");
-  if (Options::cdoVerbose && lzon) cdoPrint("Process zonal data");
-  if (Options::cdoVerbose && lmer) cdoPrint("Process meridional data");
+  if (Options::cdoVerbose && lhov) cdo_print("Process hovmoeller data");
+  if (Options::cdoVerbose && lzon) cdo_print("Process zonal data");
+  if (Options::cdoVerbose && lmer) cdo_print("Process meridional data");
   /*
   if ( lzon || lmer )
     {
       if ( operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT )
-        cdoAbort("Bounds not available for zonal/meridional data!");
+        cdo_abort("Bounds not available for zonal/meridional data!");
     }
   */
   if (lhov)
     {
-      if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdoAbort("Bounds not available hovmoeller data!");
+      if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdo_abort("Bounds not available hovmoeller data!");
     }
 
   const int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
@@ -316,13 +372,11 @@ Outputgmt(void *process)
   gridInqYvals(gridID, grid_center_lat.data());
   gridInqXvals(gridID, grid_center_lon.data());
 
-  /* Convert lat/lon units if required */
-  gridInqXunits(gridID, units);
-  grid_to_degree(units, gridsize, grid_center_lon.data(), "grid center lon");
-  gridInqYunits(gridID, units);
-  grid_to_degree(units, gridsize, grid_center_lat.data(), "grid center lat");
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID, CDI_XAXIS, gridsize, grid_center_lon.data(), "grid center lon");
+  cdo_grid_to_degree(gridID, CDI_YAXIS, gridsize, grid_center_lat.data(), "grid center lat");
 
-  int nvals = gridsize;
+  long nvals = gridsize;
   double *plon = grid_center_lon.data();
   double *plat = grid_center_lat.data();
 
@@ -336,9 +390,9 @@ Outputgmt(void *process)
       make_cyclic(grid_center_lat.data(), grid_center_lat2.data(), nlon, nlat);
       make_cyclic(grid_center_lon.data(), grid_center_lon2.data(), nlon, nlat);
 
-      for (int j = 0; j < nlat; ++j)
+      for (long j = 0; j < nlat; ++j)
         {
-          const int ij2 = j * (nlon + 1);
+          const long ij2 = j * (nlon + 1);
           grid_center_lon2[ij2 + nlon] += 360;
         }
 
@@ -347,32 +401,24 @@ Outputgmt(void *process)
       plat = grid_center_lat2.data();
     }
 
-  Varray<double> zaxis_center_lev(nlev);
-  Varray<double> zaxis_lower_lev(nlev);
-  Varray<double> zaxis_upper_lev(nlev);
+  Varray<double> zaxis_center_lev(nlev), zaxis_lower_lev(nlev), zaxis_upper_lev(nlev);
 
-  cdoZaxisInqLevels(zaxisID, zaxis_center_lev.data());
+  cdo_zaxis_inq_levels(zaxisID, zaxis_center_lev.data());
 
   if (luse_grid_corner)
     {
-      if (ncorner == 0) cdoAbort("Number of cell corners undefined!");
+      if (ncorner == 0) cdo_abort("Number of cell corners undefined!");
       const size_t nalloc = ncorner * gridsize;
       grid_corner_lat.resize(nalloc);
       grid_corner_lon.resize(nalloc);
 
-      if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
-        {
-          gridInqYbounds(gridID, grid_corner_lat.data());
-          gridInqXbounds(gridID, grid_corner_lon.data());
-        }
-      else
-        {
-          cdoAbort("Cell corner coordinates missing!");
-        }
+      if (!gridHasBounds(gridID)) cdo_abort("Cell corner coordinates missing!");
 
-      /* Note: using units from latitude instead from bounds */
-      grid_to_degree(units, ncorner * gridsize, grid_corner_lon.data(), "grid corner lon");
-      grid_to_degree(units, ncorner * gridsize, grid_corner_lat.data(), "grid corner lat");
+      gridInqYbounds(gridID, grid_corner_lat.data());
+      gridInqXbounds(gridID, grid_corner_lon.data());
+
+      cdo_grid_to_degree(gridID, CDI_XAXIS, ncorner * gridsize, grid_corner_lon.data(), "grid corner lon");
+      cdo_grid_to_degree(gridID, CDI_YAXIS, ncorner * gridsize, grid_corner_lat.data(), "grid corner lat");
 
       if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
         {
@@ -403,31 +449,32 @@ Outputgmt(void *process)
       parray = array2.data();
     }
 
-  Varray<double> uf, vf, alpha, auv;
+  Varray<double> uf, vf;
   if (operatorID == OUTPUTVECTOR)
     {
       uf.resize(gridsize);
       vf.resize(gridsize);
-      alpha.resize(gridsize);
-      auv.resize(gridsize);
     }
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
+  while (true)
     {
+      const auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
+      if (nrecs == 0) break;
+
       const auto vdate = taxisInqVdate(taxisID);
       const auto vtime = taxisInqVtime(taxisID);
-      const auto vdateString = dateToString(vdate);
-      const auto vtimeString = timeToString(vtime);
+      const auto vdateString = date_to_string(vdate);
+      const auto vtimeString = time_to_string(vtime);
 
       if (tsID == 0 && operatorID != OUTPUTTRI)
         {
           if (operatorID == OUTPUTVRML) printf("#VRML V2.0 utf8\n\n");
-#if defined(VERSION)
+#ifdef VERSION
           fprintf(stdout, "# Generated by CDO version %s\n", VERSION);
           fprintf(stdout, "#\n");
 #endif
-          fprintf(stdout, "# Operator = %s\n", cdoOperatorName(operatorID));
+          fprintf(stdout, "# Operator = %s\n", cdo_operator_name(operatorID));
           if (lhov)
             fprintf(stdout, "# Mode     = hovmoeller\n");
           else if (lzon)
@@ -439,7 +486,7 @@ Outputgmt(void *process)
 
           if (operatorID == OUTPUTVECTOR) fprintf(stdout, "# Increment = %d\n", ninc);
           fprintf(stdout, "#\n");
-          fprintf(stdout, "# Stream = %s\n", cdoGetStreamName(0));
+          fprintf(stdout, "# Stream = %s\n", cdo_get_stream_name(0));
           fprintf(stdout, "# Date   = %s\n", vdateString.c_str());
           fprintf(stdout, "# Time   = %s\n", vtimeString.c_str());
           fprintf(stdout, "# Name   = %s\n", varname);
@@ -450,16 +497,17 @@ Outputgmt(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          cdoInqRecord(streamID, &varID, &levelID);
+          int levelID;
+          cdo_inq_record(streamID, &varID, &levelID);
 
           if (varID != varID0) continue;
           if (recID > 0 && !lzon && !lmer) continue;
 
-          cdoReadRecord(streamID, array.data(), &nmiss);
+          cdo_read_record(streamID, array.data(), &nmiss);
 
           if (operatorID == OUTPUTCENTER2 && grid_is_circular) make_cyclic(array.data(), array2.data(), nlon, nlat);
 
-          const double level = zaxis_center_lev[levelID];
+          const auto level = zaxis_center_lev[levelID];
 
           if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level  = %g\n", level);
           if (lhov) fprintf(stdout, "# Timestep = %d\n", tsID + 1);
@@ -470,16 +518,14 @@ Outputgmt(void *process)
             {
               if (Options::cdoVerbose)
                 {
-                  const auto minmax = varrayMinMaxMV(gridsize, array, missval);
-                  const double minval = minmax.first;
-                  const double maxval = minmax.second;
-                  const double range = maxval - minval;
-                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
+                  const auto mm = varray_min_max_mv(gridsize, array, missval);
+                  const auto range = mm.max - mm.min;
+                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", mm.min, mm.max, range / 20);
                   fprintf(stderr, "pscontour -K -JQ0/10i -Rd -I -Cgmt.cpt data.gmt > gmtplot.ps\n");
                   fprintf(stderr, "pscoast -O -J -R -Dc -W -B40g20 >> gmtplot.ps\n");
                 }
 
-              for (int i = 0; i < nvals; i++)
+              for (long i = 0; i < nvals; i++)
                 {
                   if (grid_mask.size() && grid_mask[i] == 0) continue;
 
@@ -512,12 +558,12 @@ Outputgmt(void *process)
             }
           else if (operatorID == OUTPUTTRI)
             {
-              if (gridInqType(gridID) != GRID_CURVILINEAR) cdoAbort("Unsupported grid!");
+              if (gridInqType(gridID) != GRID_CURVILINEAR) cdo_abort("Unsupported grid!");
 
-              const int mlon = nlon - 1;
+              const long mlon = nlon - 1;
               // if ( gridIsCircular(gridID) ) mlon = nlon;
-              for (int j = 0; j < nlat - 1; ++j)
-                for (int i = 0; i < mlon; ++i)
+              for (long j = 0; j < nlat - 1; ++j)
+                for (long i = 0; i < mlon; ++i)
                   {
                     int ip1 = i + 1;
                     if (i == nlon - 1) ip1 = 0;
@@ -533,46 +579,27 @@ Outputgmt(void *process)
             }
           else if (operatorID == OUTPUTVECTOR)
             {
-              if (nrecs < 2) cdoAbort("Too few fields!");
+              if (nrecs < 2) cdo_abort("Too few fields!");
 
-              varrayCopy(gridsize, array, uf);
-              cdoInqRecord(streamID, &varID, &levelID);
-              cdoReadRecord(streamID, vf.data(), &nmiss);
+              varray_copy(gridsize, array, uf);
+              cdo_inq_record(streamID, &varID, &levelID);
+              cdo_read_record(streamID, vf.data(), &nmiss);
 
-              for (int j = 0; j < nlat; j += ninc)
-                for (int i = 0; i < nlon; i += ninc)
-                  {
-                    // compute length of velocity vector
-                    auv[IX2D(j, i, nlon)]
-                        = std::sqrt(uf[IX2D(j, i, nlon)] * uf[IX2D(j, i, nlon)] + vf[IX2D(j, i, nlon)] * vf[IX2D(j, i, nlon)]);
+              output_vector(nlon, nlat, ninc, grid_center_lon, grid_center_lat, uf, vf);
 
-                    alpha[IX2D(j, i, nlon)] = atan2(vf[IX2D(j, i, nlon)], uf[IX2D(j, i, nlon)]);
-                    alpha[IX2D(j, i, nlon)] = 90. - alpha[IX2D(j, i, nlon)] * RAD2DEG;
-
-                    if (alpha[IX2D(j, i, nlon)] < 0) alpha[IX2D(j, i, nlon)] += 360;
-                    if (alpha[IX2D(j, i, nlon)] > 360) alpha[IX2D(j, i, nlon)] -= 360;
-
-                    if (std::fabs(auv[IX2D(j, i, nlon)]) > 0)
-                      fprintf(stdout, " %g  %g  %g  %g\n", grid_center_lon[IX2D(j, i, nlon)], grid_center_lat[IX2D(j, i, nlon)],
-                              alpha[IX2D(j, i, nlon)], auv[IX2D(j, i, nlon)]);
-                  }
-
-              fprintf(stdout, "#\n");
               break;
             }
           else if (operatorID == OUTPUTVRML)
             {
-              output_vrml(nlon, nlat, gridsize, array, missval, &cpt);
+              output_vrml(nlon, nlat, gridsize, array, missval, cpt);
             }
           else if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT)
             {
               if (Options::cdoVerbose)
                 {
-                  const auto minmax = varrayMinMaxMV(gridsize, array, missval);
-                  const double minval = minmax.first;
-                  const double maxval = minmax.second;
-                  const double range = maxval - minval;
-                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
+                  const auto mm = varray_min_max_mv(gridsize, array, missval);
+                  const auto range = mm.max - mm.min;
+                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", mm.min, mm.max, range / 20);
                   fprintf(stderr, "psxy -K -JQ0/10i -Rd -L -Cgmt.cpt -m data.gmt > gmtplot.ps\n");
                   // fprintf(stderr, "psxy -K -Jx0.028id -Rd -L -Cgmt.cpt -m
                   // data.gmt > gmtplot.ps\n"); fprintf(stderr, "psxy -K
@@ -592,87 +619,22 @@ Outputgmt(void *process)
 
                   if (operatorID == OUTPUTBOUNDSCPT)
                     {
-                      int r = 0, g = 0, b = 0, n;
-
-                      if (!DBL_IS_EQUAL(array[i], missval))
-                        {
-                          for (n = 0; n < cpt.ncolors; n++)
-                            if (array[i] > cpt.lut[n].z_low && array[i] <= cpt.lut[n].z_high) break;
-
-                          if (n == cpt.ncolors)
-                            {
-                              r = cpt.bfn[0].rgb[0];
-                              g = cpt.bfn[0].rgb[1];
-                              b = cpt.bfn[0].rgb[2];
-                            }
-                          else
-                            {
-                              r = cpt.lut[n].rgb_high[0];
-                              g = cpt.lut[n].rgb_high[1];
-                              b = cpt.lut[n].rgb_high[2];
-                            }
-                        }
-                      else
-                        {
-                          r = cpt.bfn[2].rgb[0];
-                          g = cpt.bfn[2].rgb[1];
-                          b = cpt.bfn[2].rgb[2];
-                        }
-
-                      fprintf(stdout, " -G%d/%d/%d", r, g, b);
+                      output_boundscpt(array[i], missval, cpt);
                     }
 
                   fprintf(stdout, "\n");
 
                   if (lzon)
                     {
-                      double xlev[4], xlat[4];
-                      double levmin = zaxis_lower_lev[levelID];
-                      double levmax = zaxis_upper_lev[levelID];
-                      double latmin = grid_corner_lat[i * 4];
-                      double latmax = grid_corner_lat[i * 4];
-                      for (int ic = 1; ic < 4; ic++)
-                        {
-                          if (grid_corner_lat[i * 4 + ic] < latmin) latmin = grid_corner_lat[i * 4 + ic];
-                          if (grid_corner_lat[i * 4 + ic] > latmax) latmax = grid_corner_lat[i * 4 + ic];
-                        }
-                      xlev[0] = levmin;
-                      xlev[1] = levmax;
-                      xlev[2] = levmax;
-                      xlev[3] = levmin;
-                      xlat[0] = latmin;
-                      xlat[1] = latmin;
-                      xlat[2] = latmax;
-                      xlat[3] = latmax;
-                      for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlat[ic], xlev[ic]);
-                      fprintf(stdout, "   %g  %g\n", xlat[0], xlev[0]);
+                      output_zon(zaxis_lower_lev[levelID], zaxis_upper_lev[levelID], &grid_corner_lat[i * 4]);
                     }
                   else if (lmer)
                     {
-                      double xlev[4], xlon[4];
-                      double levmin = zaxis_lower_lev[levelID];
-                      double levmax = zaxis_upper_lev[levelID];
-                      double lonmin = grid_corner_lon[i * 4];
-                      double lonmax = grid_corner_lon[i * 4];
-                      for (int ic = 1; ic < 4; ic++)
-                        {
-                          if (grid_corner_lon[i * 4 + ic] < lonmin) lonmin = grid_corner_lon[i * 4 + ic];
-                          if (grid_corner_lon[i * 4 + ic] > lonmax) lonmax = grid_corner_lon[i * 4 + ic];
-                        }
-                      xlev[0] = levmin;
-                      xlev[1] = levmin;
-                      xlev[2] = levmax;
-                      xlev[3] = levmax;
-                      xlon[0] = lonmin;
-                      xlon[1] = lonmax;
-                      xlon[2] = lonmax;
-                      xlon[3] = lonmin;
-                      for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlon[ic], xlev[ic]);
-                      fprintf(stdout, "   %g  %g\n", xlon[0], xlev[0]);
+                      output_mer(zaxis_lower_lev[levelID], zaxis_upper_lev[levelID], &grid_corner_lon[i * 4]);
                     }
                   else if (lhov)
                     {
-                      cdoAbort("Implementation for hovmoeller data missing!");
+                      cdo_abort("Implementation for hovmoeller data missing!");
                     }
                   else
                     {
@@ -693,9 +655,9 @@ Outputgmt(void *process)
       tsID++;
     }
 
-  cdoStreamClose(streamID);
+  cdo_stream_close(streamID);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
