@@ -1,18 +1,11 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
   Copyright (C) 2007 Brockmann Consult
-  See COPYING file for copying and redistribution conditions.
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  Author: Ralf Quast
+          Uwe Schulzweida
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -27,34 +20,33 @@
 #include "datetime.h"
 #include "process_int.h"
 
-
 void *
 Seascount(void *process)
 {
   int64_t vdate0 = 0;
   int vtime0 = 0;
-  int nrecs;
-  int varID, levelID;
   int seas0 = 0;
   int oldmon = 0;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
-  cdoOperatorAdd("seascount", 0, 0, nullptr);
+  cdo_operator_add("seascount", 0, 0, nullptr);
 
-  const auto season_start = getSeasonStart();
+  operator_check_argc(0);
 
-  const auto streamID1 = cdoOpenRead(0);
+  const auto season_start = get_season_start();
 
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdo_open_read(0);
+
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
   const auto vlistID2 = vlistDuplicate(vlistID1);
 
   const auto taxisID1 = vlistInqTaxis(vlistID1);
   const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const auto streamID2 = cdoOpenWrite(1);
-  cdoDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdo_open_write(1);
+  cdo_def_vlist(streamID2, vlistID2);
 
   const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
@@ -66,24 +58,28 @@ Seascount(void *process)
   field.resize(gridsizemax);
 
   FieldVector2D vars1;
-  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
+  fields_from_vlist(vlistID1, vars1, FIELD_VEC);
 
   int tsID = 0;
   int otsID = 0;
   while (true)
     {
+      int nrecs = 0;
       int nsets = 0;
       bool newseas = false;
-      while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+      while (true)
         {
+          nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+          if (nrecs == 0) break;
+
           const auto vdate = taxisInqVdate(taxisID1);
           const auto vtime = taxisInqVtime(taxisID1);
 
-          const auto month = decodeMonth(vdate);
+          const auto month = decode_month(vdate);
           auto newmon = month;
           if (season_start == START_DEC && newmon == 12) newmon = 0;
 
-          const auto seas = monthToSeason(month);
+          const auto seas = month_to_season(month);
 
           if (nsets == 0)
             {
@@ -99,28 +95,30 @@ Seascount(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              cdoInqRecord(streamID1, &varID, &levelID);
+              int varID, levelID;
+              cdo_inq_record(streamID1, &varID, &levelID);
 
               if (tsID == 0)
                 {
                   recList[recID].varID = varID;
                   recList[recID].levelID = levelID;
-                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].lconst = (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT);
                 }
 
-              const size_t fieldsize = vars1[varID][levelID].size;
+              const auto fieldsize = vars1[varID][levelID].size;
 
               if (nsets == 0)
                 {
-                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec[i] = vars1[varID][levelID].missval;
+                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec_d[i] = vars1[varID][levelID].missval;
                   vars1[varID][levelID].nmiss = fieldsize;
                 }
 
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdo_read_record(streamID1, field.vec_d.data(), &field.nmiss);
+              field.size = vars1[varID][levelID].size;
               field.grid = vars1[varID][levelID].grid;
               field.missval = vars1[varID][levelID].missval;
 
-              vfarcount(vars1[varID][levelID], field);
+              field2_count(vars1[varID][levelID], field);
             }
 
           vdate0 = vdate;
@@ -133,7 +131,7 @@ Seascount(void *process)
 
       taxisDefVdate(taxisID2, vdate0);
       taxisDefVtime(taxisID2, vtime0);
-      cdoDefTimestep(streamID2, otsID);
+      cdo_def_timestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
@@ -141,18 +139,18 @@ Seascount(void *process)
 
           const auto varID = recList[recID].varID;
           const auto levelID = recList[recID].levelID;
-          cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
+          cdo_def_record(streamID2, varID, levelID);
+          cdo_write_record(streamID2, vars1[varID][levelID].vec_d.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
       otsID++;
     }
 
-  cdoStreamClose(streamID2);
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID2);
+  cdo_stream_close(streamID1);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
