@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -26,60 +16,55 @@
 #include "param_conversion.h"
 #include <mpim_grid.h>
 
-
 static int
 gentpngrid(int gridID1)
 {
-  size_t ilat, ilon, ilonr, k, kr;
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  auto nlon1 = gridInqXsize(gridID1);
-  auto nlat1 = gridInqYsize(gridID1);
+  const auto nlon2 = nlon1;
+  const auto nlat2 = nlat1 + 2;
 
-  auto nlon2 = nlon1;
-  auto nlat2 = nlat1 + 2;
+  const auto gridtype = gridInqType(gridID1);
 
-  auto gridtype = gridInqType(gridID1);
-  auto prec = gridInqDatatype(gridID1);
-
-  auto gridID2 = gridCreate(gridtype, nlon2 * nlat2);
+  const auto gridID2 = gridCreate(gridtype, nlon2 * nlat2);
   gridDefXsize(gridID2, nlon2);
   gridDefYsize(gridID2, nlat2);
 
-  gridDefDatatype(gridID2, prec);
+  int datatype = CDI_UNDEFID;
+  cdiInqKeyInt(gridID1, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
+  cdiDefKeyInt(gridID2, CDI_GLOBAL, CDI_KEY_DATATYPE, datatype);
 
-  grid_copy_attributes(gridID1, gridID2);
+  grid_copy_keys(gridID1, gridID2);
 
-  char xunits[CDI_MAX_NAME];
-  xunits[0] = 0;
-  char yunits[CDI_MAX_NAME];
-  yunits[0] = 0;
-  cdiGridInqKeyStr(gridID1, CDI_KEY_XUNITS, CDI_MAX_NAME, xunits);
-  cdiGridInqKeyStr(gridID1, CDI_KEY_YUNITS, CDI_MAX_NAME, yunits);
+  char xunits[CDI_MAX_NAME], yunits[CDI_MAX_NAME];
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(gridID1, CDI_XAXIS, CDI_KEY_UNITS, xunits, &length);
+  length = CDI_MAX_NAME;
+  cdiInqKeyString(gridID1, CDI_YAXIS, CDI_KEY_UNITS, yunits, &length);
 
-  if (gridInqXvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr))
+  if (gridHasCoordinates(gridID1))
     {
       if (gridtype == GRID_CURVILINEAR)
         {
-          Varray<double> xvals1(nlon1 * nlat1);
-          Varray<double> yvals1(nlon1 * nlat1);
-          Varray<double> xvals2(nlon2 * nlat2);
-          Varray<double> yvals2(nlon2 * nlat2);
+          Varray<double> xvals1(nlon1 * nlat1), yvals1(nlon1 * nlat1);
+          Varray<double> xvals2(nlon2 * nlat2), yvals2(nlon2 * nlat2);
 
           gridInqXvals(gridID1, xvals1.data());
           gridInqYvals(gridID1, yvals1.data());
 
-          for (ilat = 0; ilat < nlat1; ilat++)
+          for (size_t ilat = 0; ilat < nlat1; ilat++)
             {
-              for (ilon = 0; ilon < nlon1; ilon++)
+              for (size_t ilon = 0; ilon < nlon1; ilon++)
                 {
                   xvals2[(ilat + 2) * nlon1 + ilon] = xvals1[ilat * nlon1 + ilon];
                   yvals2[(ilat + 2) * nlon1 + ilon] = yvals1[ilat * nlon1 + ilon];
                 }
             }
 
-          for (ilon = 0; ilon < nlon1; ilon++)
+          for (size_t ilon = 0; ilon < nlon1; ilon++)
             {
-              ilonr = nlon1 - ilon - 1;
+              const auto ilonr = nlon1 - ilon - 1;
               xvals2[1 * nlon1 + ilon] = xvals2[2 * nlon1 + ilonr]; /* syncronise line 2 with line 3 */
               xvals2[0 * nlon1 + ilon] = xvals2[3 * nlon1 + ilonr]; /* syncronise line 1 with line 4 */
               yvals2[1 * nlon1 + ilon] = yvals2[2 * nlon1 + ilonr]; /* syncronise line 2 with line 3 */
@@ -91,14 +76,12 @@ gentpngrid(int gridID1)
         }
     }
 
-  if (gridInqXbounds(gridID1, nullptr) && gridInqYbounds(gridID1, nullptr))
+  if (gridHasBounds(gridID1))
     {
       if (gridtype == GRID_CURVILINEAR)
         {
-          Varray<double> xbounds1(4 * nlon1 * nlat1);
-          Varray<double> ybounds1(4 * nlon1 * nlat1);
-          Varray<double> xbounds2(4 * nlon2 * nlat2);
-          Varray<double> ybounds2(4 * nlon2 * nlat2);
+          Varray<double> xbounds1(4 * nlon1 * nlat1), ybounds1(4 * nlon1 * nlat1);
+          Varray<double> xbounds2(4 * nlon2 * nlat2), ybounds2(4 * nlon2 * nlat2);
 
           gridInqXbounds(gridID1, xbounds1.data());
           gridInqYbounds(gridID1, ybounds1.data());
@@ -107,21 +90,21 @@ gentpngrid(int gridID1)
             {
               gridDefNvertex(gridID2, 4);
 
-              for (ilat = 0; ilat < nlat1; ilat++)
+              for (size_t ilat = 0; ilat < nlat1; ilat++)
                 {
-                  for (ilon = 0; ilon < 4 * nlon1; ilon++)
+                  for (size_t ilon = 0; ilon < 4 * nlon1; ilon++)
                     {
                       xbounds2[4 * (ilat + 2) * nlon1 + ilon] = xbounds1[4 * ilat * nlon1 + ilon];
                       ybounds2[4 * (ilat + 2) * nlon1 + ilon] = ybounds1[4 * ilat * nlon1 + ilon];
                     }
                 }
 
-              for (ilon = 0; ilon < nlon1; ilon++)
+              for (size_t ilon = 0; ilon < nlon1; ilon++)
                 {
-                  ilonr = nlon1 - ilon - 1;
-                  for (k = 0; k < 4; ++k)
+                  const auto ilonr = nlon1 - ilon - 1;
+                  for (size_t k = 0; k < 4; ++k)
                     {
-                      kr = 3 - k;
+                      const auto kr = 3 - k;
                       xbounds2[4 * 1 * nlon1 + 4 * ilon + k] = xbounds2[4 * 2 * nlon1 + 4 * ilonr + kr];
                       xbounds2[4 * 0 * nlon1 + 4 * ilon + k] = xbounds2[4 * 3 * nlon1 + 4 * ilonr + kr];
                       ybounds2[4 * 1 * nlon1 + 4 * ilon + k] = ybounds2[4 * 2 * nlon1 + 4 * ilonr + kr];
@@ -129,9 +112,9 @@ gentpngrid(int gridID1)
                     }
                 }
               /*
-              for ( ilon = 0; ilon < 4*nlon1; ilon++ )
+              for (size_t ilon = 0; ilon < 4*nlon1; ilon++)
                 {
-                  ilonr = 4*nlon1 - ilon - 1;
+                  const auto ilonr = 4*nlon1 - ilon - 1;
                   xbounds2[4*1*nlon1 + ilon] = xbounds2[4*2*nlon1 + ilonr]; xbounds2[4*0*nlon1 + ilon] = xbounds2[4*3*nlon1 +
               ilonr];
                   ybounds2[4*1*nlon1 + ilon] = ybounds2[4*2*nlon1 + ilonr]; ybounds2[4*0*nlon1 + ilon] = ybounds2[4*3*nlon1 +
@@ -164,31 +147,29 @@ gengrid(int gridID1, long lhalo, long rhalo)
   long nmax = nlon1;
   if (lhalo < 0) nmin = -lhalo;
   if (rhalo < 0) nmax += rhalo;
-  /*
-  printf("nlon1=%d, nlon2=%d, lhalo=%d, rhalo=%d nmin=%d, nmax=%d\n",
-         nlon1, nlon2, lhalo, rhalo, nmin, nmax);
-  */
-  auto gridtype = gridInqType(gridID1);
-  auto prec = gridInqDatatype(gridID1);
+  // printf("nlon1=%d, nlon2=%d, lhalo=%d, rhalo=%d nmin=%d, nmax=%d\n", nlon1, nlon2, lhalo, rhalo, nmin, nmax);
 
-  auto gridID2 = gridCreate(gridtype, nlon2 * nlat2);
+  const auto gridtype = gridInqType(gridID1);
+
+  const auto gridID2 = gridCreate(gridtype, nlon2 * nlat2);
   gridDefXsize(gridID2, nlon2);
   gridDefYsize(gridID2, nlat2);
 
-  gridDefDatatype(gridID2, prec);
+  int datatype = CDI_UNDEFID;
+  cdiInqKeyInt(gridID1, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
+  cdiDefKeyInt(gridID2, CDI_GLOBAL, CDI_KEY_DATATYPE, datatype);
 
-  grid_copy_attributes(gridID1, gridID2);
+  grid_copy_keys(gridID1, gridID2);
 
-  char xunits[CDI_MAX_NAME];
-  xunits[0] = 0;
-  char yunits[CDI_MAX_NAME];
-  yunits[0] = 0;
-  cdiGridInqKeyStr(gridID1, CDI_KEY_XUNITS, CDI_MAX_NAME, xunits);
-  cdiGridInqKeyStr(gridID1, CDI_KEY_YUNITS, CDI_MAX_NAME, yunits);
+  char xunits[CDI_MAX_NAME], yunits[CDI_MAX_NAME];
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(gridID1, CDI_XAXIS, CDI_KEY_UNITS, xunits, &length);
+  length = CDI_MAX_NAME;
+  cdiInqKeyString(gridID1, CDI_YAXIS, CDI_KEY_UNITS, yunits, &length);
 
   if (memcmp(xunits, "degree", 6) == 0) cpi2 *= RAD2DEG;
 
-  if (gridInqXvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr))
+  if (gridHasCoordinates(gridID1))
     {
       Varray<double> xvals1, yvals1, xvals2, yvals2;
       if (gridtype == GRID_CURVILINEAR)
@@ -251,7 +232,7 @@ gengrid(int gridID1, long lhalo, long rhalo)
       gridDefYvals(gridID2, yvals2.data());
     }
 
-  if (gridInqXbounds(gridID1, nullptr) && gridInqYbounds(gridID1, nullptr))
+  if (gridHasBounds(gridID1))
     {
       Varray<double> xbounds1, ybounds1, xbounds2, ybounds2;
       if (gridtype == GRID_CURVILINEAR)
@@ -319,38 +300,38 @@ gengrid(int gridID1, long lhalo, long rhalo)
 static int
 genindexgrid(int gridID1, long &lhalo, long &rhalo)
 {
-  operatorCheckArgc(2);
+  operator_check_argc(2);
 
-  lhalo = parameter2long(cdoOperatorArgv(0));
-  rhalo = parameter2long(cdoOperatorArgv(1));
+  lhalo = parameter_to_long(cdo_operator_argv(0));
+  rhalo = parameter_to_long(cdo_operator_argv(1));
 
-  long nlon1 = gridInqXsize(gridID1);
+  const long nlon1 = gridInqXsize(gridID1);
 
   if (lhalo > nlon1)
     {
       lhalo = nlon1;
-      cdoWarning("left halo out of range. Set to %d.", lhalo);
+      cdo_warning("left halo out of range. Set to %d.", lhalo);
     }
 
   if (lhalo < 0 && -(lhalo) > nlon1 / 2)
     {
       lhalo = -nlon1 / 2;
-      cdoWarning("left halo out of range. Set to %d.", lhalo);
+      cdo_warning("left halo out of range. Set to %d.", lhalo);
     }
 
   if (rhalo > nlon1)
     {
       rhalo = nlon1;
-      cdoWarning("right halo out of range. Set to %d.", rhalo);
+      cdo_warning("right halo out of range. Set to %d.", rhalo);
     }
 
   if (rhalo < 0 && -(rhalo) > nlon1 / 2)
     {
       rhalo = -nlon1 / 2;
-      cdoWarning("right halo out of range. Set to %d.", rhalo);
+      cdo_warning("right halo out of range. Set to %d.", rhalo);
     }
 
-  int gridID2 = gengrid(gridID1, lhalo, rhalo);
+  const int gridID2 = gengrid(gridID1, lhalo, rhalo);
 
   return gridID2;
 }
@@ -387,33 +368,30 @@ tpnhalo(double *array1, int gridID1, double *array2)
 
   for (size_t ilon = 0; ilon < nlon; ilon++)
     {
-      size_t ilonr = nlon - ilon - 1;
-      array2[1 * nlon + ilon] = array2[2 * nlon + ilonr]; /* syncronise line 2 with line 3 */
-      array2[0 * nlon + ilon] = array2[3 * nlon + ilonr]; /* syncronise line 1 with line 4 */
+      const size_t ilonr = nlon - ilon - 1;
+      array2[1 * nlon + ilon] = array2[2 * nlon + ilonr];  // syncronise line 2 with line 3
+      array2[0 * nlon + ilon] = array2[3 * nlon + ilonr];  // syncronise line 1 with line 4
     }
 }
 
 void *
 Sethalo(void *process)
 {
-  int nrecs;
-  int varID, levelID;
   int gridID1 = -1, gridID2;
   int index;
-  size_t nmiss;
   long lhalo = 0, rhalo = 0;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto SETHALO = cdoOperatorAdd("sethalo", 0, 0, nullptr);
-                      cdoOperatorAdd("tpnhalo", 0, 0, nullptr);
+  const auto SETHALO = cdo_operator_add("sethalo", 0, 0, nullptr);
+                       cdo_operator_add("tpnhalo", 0, 0, nullptr);
   // clang-format on
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
 
-  const auto streamID1 = cdoOpenRead(0);
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdo_open_read(0);
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
 
   auto ngrids = vlistNgrids(vlistID1);
   int ndiffgrids = 0;
@@ -430,14 +408,14 @@ Sethalo(void *process)
     }
 
   if (gridInqType(gridID1) == GRID_GAUSSIAN_REDUCED)
-    cdoAbort("Gaussian reduced grid found. Use option -R to convert it to a regular grid!");
+    cdo_abort("Gaussian reduced grid found. Use option -R to convert it to a regular grid!");
 
-  if (index == ngrids) cdoAbort("No regular grid found!");
-  if (ndiffgrids > 0) cdoAbort("Too many different grids!");
+  if (index == ngrids) cdo_abort("No regular grid found!");
+  if (ndiffgrids > 0) cdo_abort("Too many different grids!");
 
   if (operatorID == SETHALO)
     {
-      operatorInputArg("left and right halo");
+      operator_input_arg("left and right halo");
       gridID2 = genindexgrid(gridID1, lhalo, rhalo);
     }
   else
@@ -463,10 +441,10 @@ Sethalo(void *process)
 
   const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars);
-  for (varID = 0; varID < nvars; varID++) vars[varID] = (gridID1 == vlistInqVarGrid(vlistID1, varID));
+  for (int varID = 0; varID < nvars; varID++) vars[varID] = (gridID1 == vlistInqVarGrid(vlistID1, varID));
 
-  const auto streamID2 = cdoOpenWrite(1);
-  cdoDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdo_open_write(1);
+  cdo_def_vlist(streamID2, vlistID2);
 
   const auto gridsize = gridInqSize(gridID1);
   Varray<double> array1(gridsize);
@@ -475,18 +453,23 @@ Sethalo(void *process)
   Varray<double> array2(gridsize2);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      taxisCopyTimestep(taxisID2, taxisID1);
-      cdoDefTimestep(streamID2, tsID);
+      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      cdo_taxis_copy_timestep(taxisID2, taxisID1);
+      cdo_def_timestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          cdoInqRecord(streamID1, &varID, &levelID);
+          int varID, levelID;
+          cdo_inq_record(streamID1, &varID, &levelID);
 
           if (vars[varID])
             {
-              cdoReadRecord(streamID1, array1.data(), &nmiss);
+              size_t nmiss;
+              cdo_read_record(streamID1, array1.data(), &nmiss);
 
               if (operatorID == SETHALO)
                 halo(array1.data(), gridID1, array2.data(), lhalo, rhalo);
@@ -496,21 +479,21 @@ Sethalo(void *process)
               if (nmiss)
                 {
                   const auto missval = vlistInqVarMissval(vlistID1, varID);
-                  nmiss = varrayNumMV(gridsize2, array2, missval);
+                  nmiss = varray_num_mv(gridsize2, array2, missval);
                 }
 
-              cdoDefRecord(streamID2, varID, levelID);
-              cdoWriteRecord(streamID2, array2.data(), nmiss);
+              cdo_def_record(streamID2, varID, levelID);
+              cdo_write_record(streamID2, array2.data(), nmiss);
             }
         }
 
       tsID++;
     }
 
-  cdoStreamClose(streamID2);
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID2);
+  cdo_stream_close(streamID1);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
