@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -22,13 +12,15 @@
       Smooth        smooth9         9 point smoothing
 */
 
+#include <atomic>
+#include <sstream>
+
 #include <cdi.h>
 
 #include "process_int.h"
 #include "param_conversion.h"
 #include "cdo_wtime.h"
 #include <mpim_grid.h>
-#include "gridreference.h"
 #include "constants.h"  // planet radius
 #include "pmlist.h"
 #include "cdo_options.h"
@@ -36,24 +28,27 @@
 #include "cimdOmp.h"
 #include "grid_point_search.h"
 
-enum
+enum class CurveForm
 {
-  FORM_LINEAR
+  AVG = 0,
+  LINEAR,
 };
 
-static const char *Form[] = { "linear" };
+static const char *Form[] = { "avg", "linear" };
 
 struct SmoothPoint
 {
+  double arc_radius = 0.0;
   double radius = 1.0;
   double weight0 = 0.25;
   double weightR = 0.25;
   size_t maxpoints = SIZE_MAX;
-  int form = FORM_LINEAR;
+  CurveForm form = CurveForm::LINEAR;
 };
 
-static void
-smooth(int gridID, double missval, const Varray<double> &array1, Varray<double> &array2, size_t &nmiss, const SmoothPoint &spoint)
+template <typename T>
+static size_t
+smooth(int gridID, T missval, const Varray<T> &array1, Varray<T> &array2, const SmoothPoint &spoint)
 {
   auto gridID0 = gridID;
   auto gridsize = gridInqSize(gridID);
@@ -63,20 +58,9 @@ smooth(int gridID, double missval, const Varray<double> &array1, Varray<double>
   Varray<uint8_t> mask(gridsize);
   for (size_t i = 0; i < gridsize; ++i) mask[i] = !DBL_IS_EQUAL(array1[i], missval);
 
-  auto gridtype = gridInqType(gridID);
+  gridID = generate_full_point_grid(gridID);
 
-  if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
-
-  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
-
-  if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID))
-    {
-      const auto reference = dereferenceGrid(gridID);
-      if (reference.isValid) gridID = reference.gridID;
-      if (reference.notFound) cdoAbort("Reference to source grid not found!");
-    }
-
-  if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
+  if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
 
   Varray<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, xvals.data());
@@ -94,65 +78,72 @@ smooth(int gridID, double missval, const Varray<double> &array1, Varray<double>
   bool xIsCyclic = false;
   size_t dims[2] = { gridsize, 0 };
   GridPointSearch gps;
-  gridPointSearchCreate(gps, xIsCyclic, dims, gridsize, xvals, yvals);
+  grid_point_search_create(gps, xIsCyclic, dims, gridsize, xvals, yvals);
 
-  gps.searchRadius = spoint.radius;
+  if (spoint.arc_radius > 0.0)
+    grid_point_search_set_arc_radius(gps, spoint.arc_radius);
+  else
+    grid_point_search_set_chord_radius(gps, spoint.radius);
 
-  if (Options::cdoVerbose) cdoPrint("Point search created: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds (%zu points)", cdo_get_wtime() - start, gridsize);
 
   if (Options::cdoVerbose) progress::init();
 
-  start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
   size_t naddsMin = gridsize, naddsMax = 0;
-  size_t nmissx = 0;
-  double findex = 0;
+  std::atomic<size_t> atomicCount{0}, atomicSum{0}, atomicNumMiss{0};
 
 #ifdef HAVE_OPENMP4
-#pragma omp parallel for schedule(dynamic) default(none) reduction(+ : nmissx) reduction(min : naddsMin) reduction(max : naddsMax) \
-  shared(findex, Options::cdoVerbose, knnWeights, spoint, mask, array1, array2, xvals, yvals, gps, gridsize, missval)
+#pragma omp parallel for schedule(dynamic) default(none) reduction(min : naddsMin) reduction(max : naddsMax) \
+  shared(atomicCount, atomicSum, atomicNumMiss, Options::cdoVerbose, knnWeights, spoint, mask, array1, array2, xvals, yvals, gps, gridsize, missval)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
       const auto ompthID = cdo_omp_get_thread_num();
 
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-      findex++;
-      if (Options::cdoVerbose && cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / gridsize);
+      atomicCount++;
+      if (Options::cdoVerbose && cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double)atomicCount / gridsize);
 
-      gridSearchPointSmooth(gps, xvals[i], yvals[i], knnWeights[ompthID]);
+      grid_search_point_smooth(gps, xvals[i], yvals[i], knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
       const auto nadds = knnWeights[ompthID].computeWeights(mask, spoint.radius, spoint.weight0, spoint.weightR);
       naddsMin = std::min(naddsMin, nadds);
       naddsMax = std::max(naddsMax, nadds);
-      if (nadds)
-        {
-          array2[i] = knnWeights[ompthID].arrayWeightsSum(array1);
-        }
-      else
-        {
-          nmissx++;
-          array2[i] = missval;
-        }
-    }
 
-  nmiss = nmissx;
+      array2[i] = nadds ? knnWeights[ompthID].arrayWeightsSum(array1) : missval;
+      atomicSum += nadds;
+      if (nadds == 0) atomicNumMiss++;
+    }
 
   progress::update(0, 1, 1);
 
-  if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
-  if (Options::cdoVerbose) cdoPrint("Min/Max points found: %zu/%zu", naddsMin, naddsMax);
+  size_t nmissx = atomicNumMiss;
+  size_t numPoints = atomicSum;
 
-  gridPointSearchDelete(gps);
+  if (Options::cdoVerbose) cdo_print("Point search nearest: %.2f seconds (%zu points)", cdo_get_wtime() - start, numPoints);
+  if (Options::cdoVerbose) cdo_print("Min/Max points found: %zu/%zu", naddsMin, naddsMax);
+
+  grid_point_search_delete(gps);
 
   if (gridID0 != gridID) gridDestroy(gridID);
+
+  return nmissx;
 }
 
+static void
+smooth(const Field &field1, Field &field2, const SmoothPoint &spoint)
+{
+  if (field1.memType == MemType::Float)
+    field2.nmiss = smooth(field1.grid, (float) field1.missval, field1.vec_f, field2.vec_f, spoint);
+  else
+    field2.nmiss = smooth(field1.grid, field1.missval, field1.vec_d, field2.vec_d, spoint);
+}
+
+template <typename T>
 static inline void
-smooth9_sum(size_t ij, const std::vector<uint8_t> &mask, double sfac, const Varray<double> &array, double &avg, double &divavg)
+smooth9_sum(size_t ij, const std::vector<uint8_t> &mask, double sfac, const Varray<T> &array, double &avg, double &divavg)
 {
   if (mask[ij])
     {
@@ -161,8 +152,9 @@ smooth9_sum(size_t ij, const std::vector<uint8_t> &mask, double sfac, const Varr
     }
 }
 
-static void
-smooth9(int gridID, double missval, const Varray<double> &array1, Varray<double> &array2, size_t &nmiss)
+template <typename T>
+static size_t
+smooth9(int gridID, T missval, const Varray<T> &array1, Varray<T> &array2)
 {
   const auto gridsize = gridInqSize(gridID);
   const auto nlon = gridInqXsize(gridID);
@@ -173,7 +165,7 @@ smooth9(int gridID, double missval, const Varray<double> &array1, Varray<double>
 
   for (size_t i = 0; i < gridsize; ++i) mask[i] = !DBL_IS_EQUAL(missval, array1[i]);
 
-  nmiss = 0;
+  size_t nmiss = 0;
   for (size_t i = 0; i < nlat; i++)
     {
       for (size_t j = 0; j < nlon; j++)
@@ -256,6 +248,17 @@ smooth9(int gridID, double missval, const Varray<double> &array1, Varray<double>
             }
         }
     }
+
+  return nmiss;
+}
+
+static void
+smooth9(const Field &field1, Field &field2)
+{
+  if (field1.memType == MemType::Float)
+    field2.nmiss = smooth9(field1.grid, (float) field1.missval, field1.vec_f, field2.vec_f);
+  else
+    field2.nmiss = smooth9(field1.grid, field1.missval, field1.vec_d, field2.vec_d);
 }
 
 double
@@ -264,75 +267,101 @@ radiusDegToKm(const double radiusInDeg)
   return radiusInDeg * (2.0 * PlanetRadius * M_PI) / (360.0 * 1000.0);
 }
 
-static int
-convert_form(const std::string &formstr)
+static CurveForm
+convert_curveform(const std::string &formstr)
 {
-  int form = FORM_LINEAR;
+  CurveForm form = CurveForm::LINEAR;
 
   if (formstr == "linear")
-    form = FORM_LINEAR;
+    form = CurveForm::LINEAR;
   else
-    cdoAbort("form=%s unsupported!", formstr.c_str());
+    cdo_abort("form=%s unsupported!", formstr.c_str());
 
   return form;
 }
 
 static void
-smoothGetParameter(int &xnsmooth, SmoothPoint &spoint)
+smooth_get_parameter(int &xnsmooth, SmoothPoint &spoint)
 {
-  const auto pargc = operatorArgc();
+  const auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto pargv = cdoGetOperArgv();
+      const auto pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = "SMOOTH";
-      if (kvlist.parseArguments(pargc, pargv) != 0) cdoAbort("Parse error!");
+      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
       for (const auto &kv : kvlist)
         {
           const auto &key = kv.key;
-          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key.c_str());
           const auto &value = kv.values[0];
 
           // clang-format off
-          if      (key == "nsmooth")   xnsmooth = parameter2int(value);
-          else if (key == "maxpoints") spoint.maxpoints = parameter2sizet(value);
-          else if (key == "weight0")   spoint.weight0 = parameter2double(value);
-          else if (key == "weightR")   spoint.weightR = parameter2double(value);
-          else if (key == "radius")    spoint.radius = radius_str_to_deg(value.c_str());
-          else if (key == "form")      spoint.form = convert_form(value);
-          else cdoAbort("Invalid parameter key >%s<!", key.c_str());
+          if      (key == "nsmooth")    xnsmooth = parameter_to_int(value);
+          else if (key == "maxpoints")  spoint.maxpoints = parameter_to_size_t(value);
+          else if (key == "weight0")    spoint.weight0 = parameter_to_double(value);
+          else if (key == "weightR")    spoint.weightR = parameter_to_double(value);
+          else if (key == "radius")     spoint.radius = radius_str_to_deg(value.c_str());
+          else if (key == "arc_radius") spoint.arc_radius = radius_str_to_deg(value.c_str());
+          else if (key == "form")       spoint.form = convert_curveform(value);
+          else cdo_abort("Invalid parameter key >%s<!", key.c_str());
           // clang-format on
         }
     }
 }
 
+static void
+print_parameter(int xnsmooth, const SmoothPoint &sp)
+{
+  std::stringstream outbuffer;
+
+  outbuffer << "nsmooth=" << xnsmooth;
+  outbuffer << ", maxpoints=" << sp.maxpoints;
+
+  if (sp.arc_radius > 0.0)
+    outbuffer << ", arc_radius=" << sp.arc_radius << "deg(" << radiusDegToKm(sp.arc_radius) << "km)";
+  else
+    outbuffer << ", radius=" << sp.radius << "deg(" << radiusDegToKm(sp.radius) << "km)";
+
+  outbuffer << ", form=" << Form[(int)sp.form];
+  if (sp.form == CurveForm::LINEAR)  outbuffer << ", weight0=" << sp.weight0 << ", weightR=" << sp.weightR;
+
+  cdo_print("%s", outbuffer.str());
+}
+
+static void
+check_radius_range(double radius, const char *name)
+{
+  if (radius < 0.0 || radius > 180.0) cdo_abort("%s=%g out of bounds (0-180 deg)!", name, radius);
+}
+
 void *
 Smooth(void *process)
 {
-  int nrecs;
   int xnsmooth = 1;
-  SmoothPoint spoint;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto SMOOTH  = cdoOperatorAdd("smooth",   0,   0, nullptr);
-  const auto SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, nullptr);
+  const auto SMOOTH  = cdo_operator_add("smooth",   0,   0, nullptr);
+  const auto SMOOTH9 = cdo_operator_add("smooth9",  0,   0, nullptr);
   // clang-format on
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
 
-  if (operatorID == SMOOTH) smoothGetParameter(xnsmooth, spoint);
+  SmoothPoint spoint;
+  if (operatorID == SMOOTH) smooth_get_parameter(xnsmooth, spoint);
 
-  if (spoint.radius < 0 || spoint.radius > 180) cdoAbort("%s=%g out of bounds (0-180 deg)!", "radius", spoint.radius);
+  check_radius_range(spoint.radius, "radius");
+  check_radius_range(spoint.arc_radius, "arc_radius");
 
-  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
   const auto vlistID2 = vlistDuplicate(vlistID1);
 
   const auto taxisID1 = vlistInqTaxis(vlistID1);
@@ -343,7 +372,7 @@ Smooth(void *process)
   varListInit(varList1, vlistID1);
 
   const auto nvars = vlistNvars(vlistID1);
-  std::vector<bool> varIDs(nvars);
+  std::vector<bool> varIDs(nvars, false);
 
   for (int varID = 0; varID < nvars; ++varID)
     {
@@ -360,70 +389,63 @@ Smooth(void *process)
         }
       else
         {
-          varIDs[varID] = false;
-          cdoWarning("Unsupported grid for variable %s", varList1[varID].name);
+          cdo_warning("Unsupported grid for variable %s", varList1[varID].name);
         }
     }
 
   const auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (gridsizemax < spoint.maxpoints) spoint.maxpoints = gridsizemax;
-  if (Options::cdoVerbose && operatorID == SMOOTH)
-    cdoPrint("nsmooth = %d, maxpoints = %zu, radius = %gdeg(%gkm), form = %s, weight0 = %g, weightR = %g", xnsmooth,
-             spoint.maxpoints, spoint.radius, radiusDegToKm(spoint.radius), Form[spoint.form], spoint.weight0, spoint.weightR);
+  if (Options::cdoVerbose && operatorID == SMOOTH) print_parameter(xnsmooth, spoint);
 
   spoint.radius *= DEG2RAD;
+  spoint.arc_radius *= DEG2RAD;
 
-  Varray<double> array1(gridsizemax), array2(gridsizemax);
+  Field field1, field2;
 
-  const auto streamID2 = cdoOpenWrite(1);
-  cdoDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdo_open_write(1);
+  cdo_def_vlist(streamID2, vlistID2);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      taxisCopyTimestep(taxisID2, taxisID1);
-      cdoDefTimestep(streamID2, tsID);
+      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      cdo_taxis_copy_timestep(taxisID2, taxisID1);
+      cdo_def_timestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
-          size_t nmiss;
-
-          cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, array1.data(), &nmiss);
+          cdo_inq_record(streamID1, &varID, &levelID);
+          field1.init(varList1[varID]);
+          field2.init(varList1[varID]);
+          cdo_read_record(streamID1, field1);
 
           if (varIDs[varID])
             {
-              const auto missval = varList1[varID].missval;
-              const auto gridID = varList1[varID].gridID;
-
               for (int i = 0; i < xnsmooth; ++i)
                 {
                   if (operatorID == SMOOTH)
-                    smooth(gridID, missval, array1, array2, nmiss, spoint);
+                    smooth(field1, field2, spoint);
                   else if (operatorID == SMOOTH9)
-                    smooth9(gridID, missval, array1, array2, nmiss);
+                    smooth9(field1, field2);
 
-                  varrayCopy(gridsizemax, array2, array1);
+                  field_copy(field2, field1);
                 }
+            }
 
-              cdoDefRecord(streamID2, varID, levelID);
-              cdoWriteRecord(streamID2, array2.data(), nmiss);
-            }
-          else
-            {
-              cdoDefRecord(streamID2, varID, levelID);
-              cdoWriteRecord(streamID2, array1.data(), nmiss);
-            }
+          cdo_def_record(streamID2, varID, levelID);
+          cdo_write_record(streamID2, field1);
         }
 
       tsID++;
     }
 
-  cdoStreamClose(streamID2);
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID2);
+  cdo_stream_close(streamID1);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
