@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 #include <algorithm>
@@ -21,45 +11,39 @@
 #include "cdo_options.h"
 #include "util_string.h"
 #include "cdo_vlist.h"
-#include "dmemory.h"
 #include "compare.h"
 #include "cdo_output.h"
-#include "functs.h"
-#include "array.h"
-
+#include "field.h"
 
 void
-vlistDefineTimestepType(int vlistID, int operfunc)
+vlist_define_timestep_type(int vlistID, int operfunc)
 {
   int stepType = -1;
   // clang-format off
-  if      (operfunc==func_mean)  stepType = TSTEP_AVG;
-  else if (operfunc==func_avg)   stepType = TSTEP_AVG;
-  else if (operfunc==func_sum)   stepType = TSTEP_SUM;
-  else if (operfunc==func_range) stepType = TSTEP_RANGE;
-  else if (operfunc==func_min)   stepType = TSTEP_MIN;
-  else if (operfunc==func_max)   stepType = TSTEP_MAX;
+  if      (operfunc == FieldFunc_Mean)  stepType = TSTEP_AVG;
+  else if (operfunc == FieldFunc_Avg)   stepType = TSTEP_AVG;
+  else if (operfunc == FieldFunc_Sum)   stepType = TSTEP_SUM;
+  else if (operfunc == FieldFunc_Range) stepType = TSTEP_RANGE;
+  else if (operfunc == FieldFunc_Min)   stepType = TSTEP_MIN;
+  else if (operfunc == FieldFunc_Max)   stepType = TSTEP_MAX;
   // clang-format on
 
   if (stepType != -1)
     {
-      auto nvars = vlistNvars(vlistID);
-      for (int varID = 0; varID < nvars; ++varID)
-        vlistDefVarTsteptype(vlistID, varID, stepType);
+      const auto nvars = vlistNvars(vlistID);
+      for (int varID = 0; varID < nvars; ++varID) vlistDefVarTsteptype(vlistID, varID, stepType);
     }
 }
 
 double
-cdoZaxisInqLevel(int zaxisID, int levelID)
+cdo_zaxis_inq_level(int zaxisID, int levelID)
 {
-  auto zaxistype = zaxisInqType(zaxisID);
-  auto level
-      = zaxisInqLevels(zaxisID, nullptr) ? zaxisInqLevel(zaxisID, levelID) : (zaxistype == ZAXIS_SURFACE) ? 0 : levelID + 1;
-  return level;
+  const auto zaxistype = zaxisInqType(zaxisID);
+  return zaxisInqLevels(zaxisID, nullptr) ? zaxisInqLevel(zaxisID, levelID) : (zaxistype == ZAXIS_SURFACE) ? 0.0 : levelID + 1.0;
 }
 
 int
-cdoZaxisInqLevels(int zaxisID, double *levels)
+cdo_zaxis_inq_levels(int zaxisID, double *levels)
 {
   auto size = zaxisInqLevels(zaxisID, nullptr);
 
@@ -71,9 +55,9 @@ cdoZaxisInqLevels(int zaxisID, double *levels)
         {
           size = zaxisInqSize(zaxisID);
           if (size == 1 && zaxisInqType(zaxisID) == ZAXIS_SURFACE)
-            levels[0] = 0;
+            levels[0] = 0.0;
           else
-            for (int i = 0; i < size; ++i) levels[i] = i + 1;
+            for (int i = 0; i < size; ++i) levels[i] = i + 1.0;
         }
     }
 
@@ -85,25 +69,24 @@ compare_lat_reg2d(size_t ysize, int gridID1, int gridID2)
 {
   if (ysize > 1)
     {
-      Varray<double> yvals1(ysize);
-      Varray<double> yvals2(ysize);
-
-      gridInqYvals(gridID1, &yvals1[0]);
-      gridInqYvals(gridID2, &yvals2[0]);
+      Varray<double> yvals1(ysize), yvals2(ysize);
+      const auto ny1 = gridInqYvals(gridID1, &yvals1[0]);
+      const auto ny2 = gridInqYvals(gridID2, &yvals2[0]);
+      if (ny1 == 0 || ny2 == 0) return;
 
       if (IS_EQUAL(yvals1[0], yvals2[ysize - 1]) && IS_EQUAL(yvals1[ysize - 1], yvals2[0]))
         {
           if (yvals1[0] > yvals2[0])
-            cdoWarning("Latitude orientation differ! First grid: N->S; second grid: S->N");
+            cdo_abort("Latitude orientation differ! First grid: N->S; second grid: S->N");
           else
-            cdoWarning("Latitude orientation differ! First grid: S->N; second grid: N->S");
+            cdo_abort("Latitude orientation differ! First grid: S->N; second grid: N->S");
         }
       else
         {
           for (size_t i = 0; i < ysize; ++i)
             if (std::fabs(yvals1[i] - yvals2[i]) > 3.e-5)
               {
-                cdoWarning("Grid latitudes differ!");
+                cdo_warning("Grid latitudes differ!");
                 break;
               }
         }
@@ -115,16 +98,15 @@ compare_lon_reg2d(size_t xsize, int gridID1, int gridID2)
 {
   if (xsize > 1)
     {
-      Varray<double> xvals1(xsize);
-      Varray<double> xvals2(xsize);
-
-      gridInqXvals(gridID1, &xvals1[0]);
-      gridInqXvals(gridID2, &xvals2[0]);
+      Varray<double> xvals1(xsize), xvals2(xsize);
+      const auto nx1 = gridInqXvals(gridID1, &xvals1[0]);
+      const auto nx2 = gridInqXvals(gridID2, &xvals2[0]);
+      if (nx1 == 0 || nx2 == 0) return;
 
       for (size_t i = 0; i < xsize; ++i)
         if (std::fabs(xvals1[i] - xvals2[i]) > 3.e-5)
           {
-            cdoWarning("Grid longitudes differ!");
+            cdo_warning("Grid longitudes differ!");
             break;
           }
     }
@@ -136,38 +118,39 @@ compare_grid_unstructured(int gridID1, int gridID2)
   if (gridInqXvals(gridID1, nullptr) && gridInqXvals(gridID1, nullptr) == gridInqXvals(gridID2, nullptr)
       && gridInqYvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr) == gridInqYvals(gridID2, nullptr))
     {
-      auto gridsize = gridInqSize(gridID1);
-      Varray<double> xvals1(gridsize);
-      Varray<double> xvals2(gridsize);
-      Varray<double> yvals1(gridsize);
-      Varray<double> yvals2(gridsize);
+      const auto gridsize = gridInqSize(gridID1);
+      Varray<double> xvals1(gridsize), yvals1(gridsize), xvals2(gridsize), yvals2(gridsize);
+      gridInqXvals(gridID1, xvals1.data());
+      gridInqYvals(gridID1, yvals1.data());
+      gridInqXvals(gridID2, xvals2.data());
+      gridInqYvals(gridID2, yvals2.data());
 
-      gridInqXvals(gridID1, &xvals1[0]);
-      gridInqXvals(gridID2, &xvals2[0]);
-      gridInqYvals(gridID1, &yvals1[0]);
-      gridInqYvals(gridID2, &yvals2[0]);
-
-      size_t inc = gridsize > 10000 ? gridsize / 1000 : 1;
+      // size_t inc = gridsize > 10000 ? gridsize / 1000 : 1;
+      size_t inc = 1;
       for (size_t i = 0; i < gridsize; i += inc)
         if (std::fabs(xvals1[i] - xvals2[i]) > 2.e-5 || std::fabs(yvals1[i] - yvals2[i]) > 2.e-5)
           {
-            // printf("%d %g %g %g %g %g %g\n", i, xvals1[i], xvals2[i],
-            // yvals1[i], yvals2[i], xvals1[i] - xvals2[i], yvals1[i] -
-            // yvals2[i]);
-            cdoWarning("Geographic location of some grid points differ!");
+            cdo_warning("Geographic location of some grid points differ!");
+            if (Options::cdoVerbose)
+              printf("cell=%zu x1=%g x2=%g y1=%g y2=%g dx=%g dy=%g\n", i + 1, xvals1[i], xvals2[i], yvals1[i], yvals2[i],
+                     xvals1[i] - xvals2[i], yvals1[i] - yvals2[i]);
             break;
           }
     }
 }
 
 void
-cdoCompareGrids(int gridID1, int gridID2)
+cdo_compare_grids(int gridID1, int gridID2)
 {
+  if (gridID1 == gridID2) return;
+
   // compare grids of first variable
 
-  if (gridInqType(gridID1) == gridInqType(gridID2))
+  const auto gridType1 = gridInqType(gridID1);
+  const auto gridType2 = gridInqType(gridID2);
+  if (gridType1 == gridType2)
     {
-      if (gridInqType(gridID1) == GRID_GAUSSIAN || gridInqType(gridID1) == GRID_LONLAT)
+      if (gridType1 == GRID_GAUSSIAN || gridType2 == GRID_LONLAT)
         {
           const auto xsize = gridInqXsize(gridID1);
           const auto ysize = gridInqYsize(gridID1);
@@ -175,22 +158,21 @@ cdoCompareGrids(int gridID1, int gridID2)
           if (ysize == gridInqYsize(gridID2))
             compare_lat_reg2d(ysize, gridID1, gridID2);
           else
-            cdoWarning("ysize of input grids differ!");
+            cdo_warning("ysize of input grids differ!");
 
           if (xsize == gridInqXsize(gridID2))
             compare_lon_reg2d(xsize, gridID1, gridID2);
           else
-            cdoWarning("xsize of input grids differ!");
+            cdo_warning("xsize of input grids differ!");
         }
-      else if (gridInqType(gridID1) == GRID_CURVILINEAR || gridInqType(gridID1) == GRID_UNSTRUCTURED)
+      else if (gridType1 == GRID_CURVILINEAR || gridType2 == GRID_UNSTRUCTURED)
         {
           compare_grid_unstructured(gridID1, gridID2);
         }
     }
   else if (gridInqSize(gridID1) > 1)
     {
-      cdoWarning("Grids have different types! First grid: %s; second grid: %s", gridNamePtr(gridInqType(gridID1)),
-                 gridNamePtr(gridInqType(gridID2)));
+      cdo_warning("Grids have different types! First grid: %s; second grid: %s", gridNamePtr(gridType1), gridNamePtr(gridType2));
     }
 }
 
@@ -200,8 +182,8 @@ vlistCompareNames(int vlistID1, int varID1, int vlistID2, int varID2)
   char name1[CDI_MAX_NAME], name2[CDI_MAX_NAME];
   vlistInqVarName(vlistID1, varID1, name1);
   vlistInqVarName(vlistID2, varID2, name2);
-  cstrToLowerCase(name1);
-  cstrToLowerCase(name2);
+  cstr_to_lower_case(name1);
+  cstr_to_lower_case(name2);
   return strcmp(name1, name2);
 }
 
@@ -212,12 +194,11 @@ zaxisCheckLevels(int zaxisID1, int zaxisID2)
     {
       const auto nlev1 = zaxisInqSize(zaxisID1);
       const auto nlev2 = zaxisInqSize(zaxisID2);
-      if (nlev1 != nlev2) cdoAbort("Number of levels of the input parameters do not match!");
+      if (nlev1 != nlev2) cdo_abort("Number of levels of the input parameters do not match!");
 
-      Varray<double> lev1(nlev1);
-      Varray<double> lev2(nlev1);
-      cdoZaxisInqLevels(zaxisID1, &lev1[0]);
-      cdoZaxisInqLevels(zaxisID2, &lev2[0]);
+      Varray<double> lev1(nlev1), lev2(nlev1);
+      cdo_zaxis_inq_levels(zaxisID1, &lev1[0]);
+      cdo_zaxis_inq_levels(zaxisID2, &lev2[0]);
 
       auto ldiffer = false;
       for (int i = 0; i < nlev1; ++i)
@@ -237,9 +218,9 @@ zaxisCheckLevels(int zaxisID1, int zaxisID2)
               }
 
           if (ldiffer)
-            cdoWarning("Input parameters have different levels!");
+            cdo_warning("Input parameters have different levels!");
           else
-            cdoWarning("Z-axis orientation differ!");
+            cdo_warning("Z-axis orientation differ!");
 
           return 1;
         }
@@ -256,8 +237,7 @@ vlistCheckNames(int vlistID1, int vlistID2)
 
   // std::vector<std::array<char,CDI_MAX_NAME>> names1(nvars); C++14?
   // std::vector<std::array<char,CDI_MAX_NAME>> names2(nvars);
-  std::vector<std::vector<char>> names1(nvars);
-  std::vector<std::vector<char>> names2(nvars);
+  std::vector<std::vector<char>> names1(nvars), names2(nvars);
   for (varID = 0; varID < nvars; varID++) names1[varID].resize(CDI_MAX_NAME);
   for (varID = 0; varID < nvars; varID++) names2[varID].resize(CDI_MAX_NAME);
   for (varID = 0; varID < nvars; varID++) vlistInqVarName(vlistID1, varID, names1[varID].data());
@@ -267,22 +247,63 @@ vlistCheckNames(int vlistID1, int vlistID2)
   std::sort(names2.begin(), names2.end());
 
   for (varID = 0; varID < nvars; varID++)
-    if (!cstrIsEqual(names1[varID].data(), names2[varID].data())) break;
+    if (!cdo_cmpstr(names1[varID].data(), names2[varID].data())) break;
 
-  if (varID == nvars) cdoPrint("Use CDO option --sortname to sort the parameter by name (NetCDF only)!");
+  if (varID == nvars) cdo_print("Use CDO option --sortname to sort the parameter by name (NetCDF only)!");
 }
 
 void
-vlistCompare(int vlistID1, int vlistID2, int flag)
+vlistPrintMissingVars(int vlistID1, int vlistID2)
+{
+  int varID1, varID2;
+  const auto nvars1 = vlistNvars(vlistID1);
+  const auto nvars2 = vlistNvars(vlistID2);
+
+  std::vector<std::vector<char>> names1(nvars1), names2(nvars2);
+  for (varID1 = 0; varID1 < nvars1; varID1++) names1[varID1].resize(CDI_MAX_NAME);
+  for (varID2 = 0; varID2 < nvars2; varID2++) names2[varID2].resize(CDI_MAX_NAME);
+  for (varID1 = 0; varID1 < nvars1; varID1++) vlistInqVarName(vlistID1, varID1, names1[varID1].data());
+  for (varID2 = 0; varID2 < nvars2; varID2++) vlistInqVarName(vlistID2, varID2, names2[varID2].data());
+
+  if (nvars1 > nvars2)
+    {
+      for (varID1 = 0; varID1 < nvars1; varID1++)
+        {
+          for (varID2 = 0; varID2 < nvars2; varID2++)
+            {
+              if (cdo_cmpstr(names1[varID1].data(), names2[varID2].data())) break;
+            }
+          if (varID2 == nvars2) cdo_print("Variable %s not found in second input stream!", names1[varID1].data());
+        }
+    }
+  else
+    {
+      for (varID2 = 0; varID2 < nvars2; varID2++)
+        {
+          for (varID1 = 0; varID1 < nvars1; varID1++)
+            {
+              if (cdo_cmpstr(names1[varID1].data(), names2[varID2].data())) break;
+            }
+          if (varID1 == nvars1) cdo_print("Variable %s not found in first input stream!", names2[varID2].data());
+        }
+    }
+}
+
+void
+vlist_compare(int vlistID1, int vlistID2, int flag)
 {
   auto lchecknames = false;
 
   const auto nvars = vlistNvars(vlistID1);
 
-  if (nvars != vlistNvars(vlistID2)) cdoAbort("Input streams have different number of variables per timestep!");
+  if (nvars != vlistNvars(vlistID2))
+    {
+      vlistPrintMissingVars(vlistID1, vlistID2);
+      cdo_abort("Input streams have different number of variables per timestep!");
+    }
 
   if (vlistNrecs(vlistID1) != vlistNrecs(vlistID2))
-    cdoAbort("Input streams have different number of %s per timestep!", nvars == 1 ? "layers" : "records");
+    cdo_abort("Input streams have different number of %s per timestep!", nvars == 1 ? "layers" : "records");
 
   for (int varID = 0; varID < nvars; varID++)
     {
@@ -292,7 +313,7 @@ vlistCompare(int vlistID1, int vlistID2, int flag)
             {
               if (vlistCompareNames(vlistID1, varID, vlistID2, varID) != 0)
                 {
-                  cdoWarning("Input streams have different parameter names!");
+                  cdo_warning("Input streams have different parameter names!");
                   lchecknames = true;
                   flag -= CMP_NAME;
                   //    break;
@@ -306,7 +327,7 @@ vlistCompare(int vlistID1, int vlistID2, int flag)
             {
               char name[CDI_MAX_NAME];
               vlistInqVarName(vlistID1, varID, name);
-              cdoAbort("Grid size of the input parameter %s do not match!", name);
+              cdo_abort("Grid size of the input parameter %s do not match!", name);
             }
         }
 
@@ -322,33 +343,33 @@ vlistCompare(int vlistID1, int vlistID2, int flag)
     {
       const auto gridID1 = vlistInqVarGrid(vlistID1, 0);
       const auto gridID2 = vlistInqVarGrid(vlistID2, 0);
-      cdoCompareGrids(gridID1, gridID2);
+      if (gridID1 != gridID2) cdo_compare_grids(gridID1, gridID2);
     }
 
   if (lchecknames) vlistCheckNames(vlistID1, vlistID2);
 }
 
 int
-vlistCompareX(int vlistID1, int vlistID2, int flag)
+vlist_compare_x(int vlistID1, int vlistID2, int flag)
 {
   const auto nvars = vlistNvars(vlistID1);
   const auto nvars2 = vlistNvars(vlistID2);
   const auto nlevels2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, 0));
 
-  if (nvars2 != 1) cdoAbort("Internal problem, vlistCompareX() called with unexpected vlistID2 argument!");
+  if (nvars2 != 1) cdo_abort("Internal problem, vlist_compare_x() called with unexpected vlistID2 argument!");
 
   for (int varID = 0; varID < nvars; varID++)
     {
       if (flag & CMP_GRIDSIZE)
         {
           if (gridInqSize(vlistInqVarGrid(vlistID1, varID)) != gridInqSize(vlistInqVarGrid(vlistID2, 0)))
-            cdoAbort("Grid size of the input parameters do not match!");
+            cdo_abort("Grid size of the input parameters do not match!");
         }
 
       if (flag & CMP_NLEVEL)
         {
           if ((zaxisInqSize(vlistInqVarZaxis(vlistID1, varID)) != nlevels2) && nlevels2 > 1)
-            cdoAbort("Number of levels of the input parameters do not match!");
+            cdo_abort("Number of levels of the input parameters do not match!");
         }
     }
 
@@ -356,21 +377,20 @@ vlistCompareX(int vlistID1, int vlistID2, int flag)
     {
       const auto gridID1 = vlistInqVarGrid(vlistID1, 0);
       const auto gridID2 = vlistInqVarGrid(vlistID2, 0);
-      cdoCompareGrids(gridID1, gridID2);
+      if (gridID1 != gridID2) cdo_compare_grids(gridID1, gridID2);
     }
 
   return nlevels2;
 }
 
 void
-vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &mapOfVarIDs)
+vlist_map(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &mapOfVarIDs)
 {
   int varID1, varID2;
   const auto nvars1 = vlistNvars(vlistID1);
   const auto nvars2 = vlistNvars(vlistID2);
 
-  std::vector<std::vector<char>> names1(nvars1);
-  std::vector<std::vector<char>> names2(nvars2);
+  std::vector<std::vector<char>> names1(nvars1), names2(nvars2);
   for (varID1 = 0; varID1 < nvars1; varID1++) names1[varID1].resize(CDI_MAX_NAME);
   for (varID2 = 0; varID2 < nvars2; varID2++) names2[varID2].resize(CDI_MAX_NAME);
   for (varID1 = 0; varID1 < nvars1; varID1++) vlistInqVarName(vlistID1, varID1, names1[varID1].data());
@@ -382,11 +402,11 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           for (varID1 = 0; varID1 < nvars1; varID1++)
             {
-              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;
+              if (cdo_cmpstr(names1[varID1].data(), names2[varID2].data())) break;
             }
           if (varID1 == nvars1)
             {
-              cdoAbort("Variable %s not found!", names2[varID2].data());
+              cdo_abort("Variable %s not found!", names2[varID2].data());
             }
           else
             {
@@ -400,12 +420,12 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           for (varID2 = 0; varID2 < nvars2; varID2++)
             {
-              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;
+              if (cdo_cmpstr(names1[varID1].data(), names2[varID2].data())) break;
             }
           if (varID2 == nvars2)
             {
               if (mapflag == 3) continue;
-              cdoAbort("Variable %s not found!", names1[varID1].data());
+              cdo_abort("Variable %s not found!", names1[varID1].data());
             }
           else
             {
@@ -414,14 +434,14 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         }
     }
 
-  if (mapOfVarIDs.empty()) cdoAbort("No variable found!");
+  if (mapOfVarIDs.empty()) cdo_abort("No variable found!");
 
   if (Options::cdoVerbose)
     for (varID1 = 0; varID1 < nvars1; varID1++)
       {
         const auto &it = mapOfVarIDs.find(varID1);
         if (it != mapOfVarIDs.end())
-          cdoPrint("Variable %d:%s mapped to %d:%s", varID1, names1[varID1].data(), it->second, names2[it->second].data());
+          cdo_print("Variable %d:%s mapped to %d:%s", varID1, names1[varID1].data(), it->second, names2[it->second].data());
       }
 
   if (mapOfVarIDs.size() > 1)
@@ -430,7 +450,7 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
       for (auto it = ++mapOfVarIDs.begin(); it != mapOfVarIDs.end(); ++it)
         {
           if (it->second < varID2)
-            cdoAbort("Variable names must be sorted, use CDO option --sortname to sort the parameter by name (NetCDF only)!");
+            cdo_abort("Variable names must be sorted, use CDO option --sortname to sort the parameter by name (NetCDF only)!");
 
           varID2 = it->second;
         }
@@ -444,7 +464,7 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
       if (flag & CMP_GRIDSIZE)
         {
           if (gridInqSize(vlistInqVarGrid(vlistID1, varID1)) != gridInqSize(vlistInqVarGrid(vlistID2, varID2)))
-            cdoAbort("Grid size of the input parameters do not match!");
+            cdo_abort("Grid size of the input parameters do not match!");
         }
 
       if (flag & CMP_NLEVEL)
@@ -458,32 +478,26 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           const auto gridID1 = vlistInqVarGrid(vlistID1, varID1);
           const auto gridID2 = vlistInqVarGrid(vlistID2, varID2);
-          cdoCompareGrids(gridID1, gridID2);
+          if (gridID1 != gridID2) cdo_compare_grids(gridID1, gridID2);
         }
     }
 }
 
 bool
-vlistIsSzipped(int vlistID)
+vlist_is_szipped(int vlistID)
 {
-  auto lszip = false;
-
   const auto nvars = vlistNvars(vlistID);
   for (int varID = 0; varID < nvars; varID++)
     {
-      const int comptype = vlistInqVarCompType(vlistID, varID);
-      if (comptype == CDI_COMPRESS_SZIP)
-        {
-          lszip = true;
-          break;
-        }
+      const auto comptype = vlistInqVarCompType(vlistID, varID);
+      if (comptype == CDI_COMPRESS_SZIP) return true;
     }
 
-  return lszip;
+  return false;
 }
 
 int
-vlistInqNWPV(int vlistID, int varID)
+vlist_inq_nwpv(int vlistID, int varID)
 {
   const auto datatype = vlistInqVarDatatype(vlistID, varID);
   // number of words per value; real:1  complex:2
@@ -512,23 +526,22 @@ vlist_check_gridsize(int vlistID)
 
   if (lerror)
     {
-      cdoPrint("This operator requires all variables on the same horizontal grid.");
-      cdoPrint("Horizontal grids found:");
+      cdo_print("This operator requires all variables on the same horizontal grid.");
+      cdo_print("Horizontal grids found:");
       for (int index = 0; index < ngrids; ++index)
         {
           const auto gridID = vlistGrid(vlistID, index);
-          cdoPrint("  grid=%d  type=%s  points=%zu", index + 1, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID));
+          cdo_print("  grid=%d  type=%s  points=%zu", index + 1, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID));
         }
-      cdoAbort("The input stream contains variables on different horizontal grids!");
+      cdo_abort("The input stream contains variables on different horizontal grids!");
     }
 
   return ngp;
 }
 
-double *
-vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlevf, int *rnhlevh)
+void
+vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlevf, int *rnhlevh, Varray<double> &vct)
 {
-  double *vct = nullptr;
   int zaxisIDh = -1;
   int nhlev = 0, nhlevf = 0, nhlevh = 0;
   int nvct = 0;
@@ -544,14 +557,14 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
       auto zaxistype = zaxisInqType(zaxisID);
 
       if (Options::cdoVerbose)
-        cdoPrint("ZAXIS_HYBRID = %d ZAXIS_HYBRID_HALF=%d nlevel=%d mono_level=%d", zaxisInqType(zaxisID) == ZAXIS_HYBRID,
-                 zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF, nlevel, mono_level);
+        cdo_print("ZAXIS_HYBRID=%d ZAXIS_HYBRID_HALF=%d nlevel=%d mono_level=%d", zaxisInqType(zaxisID) == ZAXIS_HYBRID,
+                  zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF, nlevel, mono_level);
 
       if ((zaxistype == ZAXIS_HYBRID || zaxistype == ZAXIS_HYBRID_HALF) && nlevel > 1 && !mono_level)
         {
-          int l;
           Varray<double> level(nlevel);
-          cdoZaxisInqLevels(zaxisID, &level[0]);
+          cdo_zaxis_inq_levels(zaxisID, &level[0]);
+          int l;
           for (l = 0; l < nlevel; l++)
             {
               if ((l + 1) != (int) (level[l] + 0.5)) break;
@@ -572,12 +585,12 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
                   nhlevf = nhlev;
                   nhlevh = nhlevf + 1;
 
-                  vct = (double *) Malloc(nvct * sizeof(double));
-                  zaxisInqVct(zaxisID, vct);
+                  vct.resize(nvct);
+                  zaxisInqVct(zaxisID, vct.data());
                   if (Options::cdoVerbose)
-                    cdoPrint("Detected half-level model definition : nlevel == (nvct/2 - 1) (nlevel: %d, nvct: %d, nhlevf: %d, "
-                             "nhlevh: %d) ",
-                             nlevel, nvct, nhlevf, nhlevh);
+                    cdo_print("Detected half-level model definition : nlevel == (nvct/2 - 1) (nlevel: %d, nvct: %d, nhlevf: %d, "
+                              "nhlevh: %d) ",
+                              nlevel, nvct, nhlevf, nhlevh);
                 }
             }
           else if (nlevel == (nvct / 2))
@@ -590,10 +603,10 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
                   nhlevf = nhlev - 1;
                   nhlevh = nhlev;
 
-                  vct = (double *) Malloc(nvct * sizeof(double));
-                  zaxisInqVct(zaxisID, vct);
+                  vct.resize(nvct);
+                  zaxisInqVct(zaxisID, vct.data());
                   if (Options::cdoVerbose)
-                    cdoPrint(
+                    cdo_print(
                         "Detected full-level model definition : nlevel == (nvct/2) (nlevel: %d, nvct: %d, nhlevf: %d, nhlevh: %d) ",
                         nlevel, nvct, nhlevf, nhlevh);
                 }
@@ -602,12 +615,10 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
             {
               if (!lhavevct)
                 {
-                  int vctsize;
-                  int voff = 4;
-
                   Varray<double> rvct(nvct);
-                  zaxisInqVct(zaxisID, &rvct[0]);
+                  zaxisInqVct(zaxisID, rvct.data());
 
+                  int voff = 4;
                   if ((int) (rvct[0] + 0.5) == 100000 && rvct[voff] < rvct[voff + 1])
                     {
                       lhavevct = true;
@@ -616,10 +627,10 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
                       nhlevf = nhlev;
                       nhlevh = nhlev + 1;
 
-                      vctsize = 2 * nhlevh;
-                      vct = (double *) Malloc(vctsize * sizeof(double));
+                      int vctsize = 2 * nhlevh;
+                      vct.resize(vctsize);
 
-                      /* calculate VCT for LM */
+                      // calculate VCT for LM
 
                       for (int i = 0; i < vctsize / 2; i++)
                         {
@@ -651,8 +662,6 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
   *rnhlev = nhlev;
   *rnhlevf = nhlevf;
   *rnhlevh = nhlevh;
-
-  return vct;
 }
 
 void
@@ -690,51 +699,27 @@ vlist_change_hybrid_zaxis(int vlistID1, int vlistID2, int zaxisID1, int zaxisID2
 int
 vlist_get_psvarid(int vlistID, int zaxisID)
 {
-  int psvarid = -1;
-  char name[CDI_MAX_NAME];
   char psname[CDI_MAX_NAME];
-  psname[0] = 0;
-  cdiZaxisInqKeyStr(zaxisID, CDI_KEY_PSNAME, CDI_MAX_NAME, psname);
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_PSNAME, psname, &length);
 
   if (psname[0])
     {
+      char name[CDI_MAX_NAME];
       const auto nvars = vlistNvars(vlistID);
       for (int varID = 0; varID < nvars; ++varID)
         {
           vlistInqVarName(vlistID, varID, name);
-          if (cstrIsEqual(name, psname))
-            {
-              psvarid = varID;
-              break;
-            }
+          if (cdo_cmpstr(name, psname)) return varID;
         }
-      if (Options::cdoVerbose && psvarid == -1) cdoWarning("Surface pressure variable not found - %s", psname);
+      if (Options::cdoVerbose) cdo_warning("Surface pressure variable not found - %s", psname);
     }
 
-  return psvarid;
+  return -1;
 }
 
-void
-varListInit(VarList &vl, int vlistID)
-{
-  const auto nvars = vlistNvars(vlistID);
-  vl.resize(nvars);
-
-  for (int varID = 0; varID < nvars; ++varID)
-    {
-      vl[varID].gridID = vlistInqVarGrid(vlistID, varID);
-      vl[varID].zaxisID = vlistInqVarZaxis(vlistID, varID);
-      vl[varID].gridsize = gridInqSize(vl[varID].gridID);
-      vl[varID].nlevels = zaxisInqSize(vl[varID].zaxisID);
-      vl[varID].datatype = vlistInqVarDatatype(vlistID, varID);
-      vl[varID].missval = vlistInqVarMissval(vlistID, varID);
-      vl[varID].timetype = vlistInqVarTimetype(vlistID, varID);
-    }
-}
-
-
 int
-vlistGetFirstSpectralGrid(int vlistID)
+vlist_get_first_spectral_grid(int vlistID)
 {
   // find first spectral grid
   const auto ngrids = vlistNgrids(vlistID);
@@ -748,7 +733,7 @@ vlistGetFirstSpectralGrid(int vlistID)
 }
 
 int
-vlistGetFirstGaussianGrid(int vlistID)
+vlist_get_first_gaussian_grid(int vlistID)
 {
   // find first gaussian grid
   const auto ngrids = vlistNgrids(vlistID);
@@ -762,7 +747,7 @@ vlistGetFirstGaussianGrid(int vlistID)
 }
 
 int
-vlistGetFirstFourierGrid(int vlistID)
+vlist_get_first_fourier_grid(int vlistID)
 {
   // find first fourier grid
   const auto ngrids = vlistNgrids(vlistID);
@@ -774,3 +759,18 @@ vlistGetFirstFourierGrid(int vlistID)
 
   return -1;
 }
+
+void
+cdo_check_missval(double missval, const char *varname)
+{
+  if (DBL_IS_EQUAL(0, missval) || DBL_IS_EQUAL(1, missval))
+    {
+      static bool lwarn = true;
+      if (lwarn)
+        {
+          lwarn = false;
+          cdo_warning("Variable %s has a missing value of %g, this can lead to incorrect results with this operator!", varname,
+                      missval);
+        }
+    }
+}
