@@ -1,18 +1,11 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
+
+#include <atomic>
 #include <vector>
 
 #include <cdi.h>
@@ -24,11 +17,10 @@
 #include "progress.h"
 #include "cimdOmp.h"
 
-
 static void
 lonlat_to_xyz(double lon, double lat, double *xyz)
 {
-  double coslat = std::cos(lat);
+  const auto coslat = std::cos(lat);
   xyz[0] = coslat * std::cos(lon);
   xyz[1] = coslat * std::sin(lon);
   xyz[2] = std::sin(lat);
@@ -263,8 +255,8 @@ mod_huiliers_area2(int num_corners, double *cell_corner_lon, double *cell_corner
 static void
 getLonLatCorner(size_t nx, size_t idx, const double *grid_corner_lon, const double *grid_corner_lat, double *lons, double *lats)
 {
-  size_t j = idx / nx;
-  size_t i = idx - j * nx;
+  const auto j = idx / nx;
+  const auto i = idx - j * nx;
 
   lons[0] = grid_corner_lon[2 * i];
   lons[1] = grid_corner_lon[2 * i + 1];
@@ -294,10 +286,9 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
 
   int status = 0;
 
-  if (gridInqXvals(gridID, nullptr) == 0 && gridInqYvals(gridID, nullptr) == 0 && gridInqXbounds(gridID, nullptr) == 0
-      && gridInqYbounds(gridID, nullptr) == 0)
+  if (!gridHasCoordinates(gridID) && !gridHasBounds(gridID))
     {
-      cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
+      cdo_warning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
       return 1;
     }
 
@@ -307,13 +298,13 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
   auto nlat = gridInqYsize(gridID);
   if (!nlon) nlon = 1;
   if (!nlat) nlat = 1;
-  std::vector<double> grid_corner_lon(nlon * 2);
-  std::vector<double> grid_corner_lat(nlat * 2);
+  std::vector<double> grid_corner_lon(nlon * 2), grid_corner_lat(nlat * 2);
 
   if (gridInqXbounds(gridID, nullptr))
     {
       gridInqXbounds(gridID, grid_corner_lon.data());
-      gridInqXunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
   else if (lweights && nlon == 1)
     {
@@ -330,13 +321,15 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
       std::vector<double> grid_center_lon(nlon);
       gridInqXvals(gridID, grid_center_lon.data());
       grid_gen_bounds(nlon, grid_center_lon, grid_corner_lon);
-      gridInqXunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
 
   if (gridInqYbounds(gridID, nullptr))
     {
       gridInqYbounds(gridID, grid_corner_lat.data());
-      gridInqYunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
   else if (lweights && nlat == 1)
     {
@@ -354,34 +347,32 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
       gridInqYvals(gridID, grid_center_lat.data());
       grid_gen_bounds(nlat, grid_center_lat, grid_corner_lat);
       grid_check_lat_borders(nlat * 2, grid_corner_lat.data());
-      gridInqYunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
 
   grid_to_radian(unitstr, nlon * 2, grid_corner_lon.data(), "grid cell corner longitudes");
   grid_to_radian(unitstr, nlat * 2, grid_corner_lat.data(), "grid cell corner latitudes");
 
   int nv = 4;
-  double findex = 0;
+  std::atomic<size_t> atomicCount{0};
 
-  Progress::init();
+  progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, nlon, findex, area, nv, grid_corner_lon, grid_corner_lat)
+#pragma omp parallel for default(none) shared(gridsize, nlon, atomicCount, area, nv, grid_corner_lon, grid_corner_lat)
 #endif
-  for (size_t i = 0; i < gridsize; ++i)
+  for (size_t i = 0; i < gridsize; i++)
     {
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-      findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
+      atomicCount++;
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double)atomicCount / gridsize);
 
       double lons[4], lats[4];
       getLonLatCorner(nlon, i, grid_corner_lon.data(), grid_corner_lat.data(), lons, lats);
       area[i] = mod_huiliers_area(nv, lons, lats);
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   return status;
 }
@@ -400,7 +391,7 @@ gridGenAreaUnstruct(int gridID, double *area)
   bool lgriddestroy = false;
 
   auto gridsize = gridInqSize(gridID);
-  int gridtype = gridInqType(gridID);
+  auto gridtype = gridInqType(gridID);
 
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
@@ -416,20 +407,18 @@ gridGenAreaUnstruct(int gridID, double *area)
         }
     }
 
-  if (gridtype == GRID_UNSTRUCTURED)
+  if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID))
     {
-      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+      const auto reference = dereferenceGrid(gridID);
+      if (reference.exists)
         {
-          if (gridInqNumber(gridID) > 0)
+          if (!reference.isValid)
             {
-              lgriddestroy = true;
-              gridID = referenceToGrid(gridID);
-              if (gridID == -1)
-                {
-                  cdoWarning("Reference to source grid not found!");
-                  return 1;
-                }
+              cdo_warning("Reference to source grid not found!");
+              return 1;
             }
+          gridID = reference.gridID;
+          lgriddestroy = true;
         }
     }
 
@@ -437,66 +426,49 @@ gridGenAreaUnstruct(int gridID, double *area)
 
   size_t nv = (gridtype == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+  if (!gridHasCoordinates(gridID))
     {
-      cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
+      cdo_warning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
       return 1;
     }
 
   if (nv == 0)
     {
-      cdoWarning("Computation of grid cell area weights failed, grid cell corner coordinates missing!");
+      cdo_warning("Computation of grid cell area weights failed, grid cell corner coordinates missing!");
       return 1;
     }
 
-  char xunitstr[CDI_MAX_NAME];
-  char yunitstr[CDI_MAX_NAME];
-  gridInqXunits(gridID, xunitstr);
-  gridInqYunits(gridID, yunitstr);
-
-  std::vector<double> grid_center_lon(gridsize);
-  std::vector<double> grid_center_lat(gridsize);
-
+  std::vector<double> grid_center_lon(gridsize), grid_center_lat(gridsize);
   gridInqXvals(gridID, grid_center_lon.data());
   gridInqYvals(gridID, grid_center_lat.data());
 
-  std::vector<double> grid_corner_lon(nv * gridsize);
-  std::vector<double> grid_corner_lat(nv * gridsize);
+  std::vector<double> grid_corner_lon(nv * gridsize), grid_corner_lat(nv * gridsize);
 
-  if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
-    {
-      gridInqXbounds(gridID, grid_corner_lon.data());
-      gridInqYbounds(gridID, grid_corner_lat.data());
-    }
-  else
-    {
-      status = 1;
-      return status;
-    }
+  if (!gridHasBounds(gridID)) return 1;
 
-  grid_to_radian(xunitstr, gridsize, grid_center_lon.data(), "grid1 center longitudes");
-  grid_to_radian(xunitstr, gridsize * nv, grid_corner_lon.data(), "grid1 corner longitudes");
+  gridInqXbounds(gridID, grid_corner_lon.data());
+  gridInqYbounds(gridID, grid_corner_lat.data());
 
-  grid_to_radian(yunitstr, gridsize, grid_center_lat.data(), "grid1 center latitudes");
-  grid_to_radian(yunitstr, gridsize * nv, grid_corner_lat.data(), "grid1 corner latitudes");
+  cdo_grid_to_radian(gridID, CDI_XAXIS, gridsize, grid_center_lon.data(), "grid1 center longitudes");
+  cdo_grid_to_radian(gridID, CDI_XAXIS, gridsize * nv, grid_corner_lon.data(), "grid1 corner longitudes");
+
+  cdo_grid_to_radian(gridID, CDI_YAXIS, gridsize, grid_center_lat.data(), "grid1 center latitudes");
+  cdo_grid_to_radian(gridID, CDI_YAXIS, gridsize * nv, grid_corner_lat.data(), "grid1 corner latitudes");
 
   if (lgriddestroy) gridDestroy(gridID);
 
-  double findex = 0;
+  std::atomic<size_t> atomicCount{0};
 
-  Progress::init();
+  progress::init();
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-    shared(gridsize, nv, findex, area, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat)
+    shared(gridsize, nv, atomicCount, area, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-      findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
+      atomicCount++;
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double)atomicCount / gridsize);
 
       if (nv <= 4)
         area[i] = mod_huiliers_area(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv]);
@@ -505,7 +477,7 @@ gridGenAreaUnstruct(int gridID, double *area)
             = mod_huiliers_area2(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv], grid_center_lon[i], grid_center_lat[i]);
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   return status;
 }
@@ -517,38 +489,33 @@ gridGenArea(const int gridID, double *area)
 
   const auto gridsize = gridInqSize(gridID);
   const auto gridtype = gridInqType(gridID);
-  const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
 
   if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)
     {
       bool lweights = false;
       status = gridGenAreaReg2D(gridID, area, lweights);
     }
-  else if (projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC
-           || projtype == CDI_PROJ_STERE || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
+  else if (gridProjIsSupported(gridID) || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
            || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       status = gridGenAreaUnstruct(gridID, area);
     }
   else
     {
-      cdoAbort("Internal error! Unsupported gridtype: %s", gridNamePtr(gridtype));
+      cdo_abort("Internal error! Unsupported gridtype: %s", gridNamePtr(gridtype));
     }
 
   if (gridVerbose)
     {
       double total_area = 0;
       for (size_t i = 0; i < gridsize; ++i) total_area += area[i];
-      cdoPrint("Total area = %g steradians", total_area);
+      cdo_print("Total area = %g steradians", total_area);
     }
 
   if (gridsize < 20)
     {
       double total_area = 0;
       for (size_t i = 0; i < gridsize; ++i) total_area += area[i];
-      int nzero = 0;
-      for (size_t i = 0; i < gridsize; ++i)
-        if (IS_EQUAL(area[i], 0.)) nzero++;
       if (IS_EQUAL(total_area, 0.)) status = 2;
     }
 
