@@ -1,19 +1,10 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
+
 #include <cdi.h>
 #include "cdi_uuid.h"
 
@@ -113,7 +104,6 @@ grid_print_attributes(FILE *fp, int gridID)
   for (int iatt = 0; iatt < natts; ++iatt)
     {
       cdiInqAtt(cdiID, varID, iatt, attname, &atttype, &attlen);
-
       if (attlen == 0) continue;
 
       if (cdo_cmpstr(attname, "grid_mapping_name")) continue;
@@ -147,10 +137,10 @@ grid_print_attributes(FILE *fp, int gridID)
           fprintf(fp, "%s =", attname);
           if (atttype == CDI_DATATYPE_FLT32)
             for (int i = 0; i < attlen; ++i)
-              fprintf(fp, " %sf", doubleToAttstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
+              fprintf(fp, " %sf", double_to_att_str(Options::CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
           else
             for (int i = 0; i < attlen; ++i)
-              fprintf(fp, " %s", doubleToAttstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
+              fprintf(fp, " %s", double_to_att_str(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
           fprintf(fp, "\n");
         }
     }
@@ -173,9 +163,10 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
   const auto ysize = gridInqYsize(gridID);
   const auto type = gridInqType(gridID);
   const auto nvertex = gridInqNvertex(gridID);
-  const auto datatype = gridInqDatatype(gridID);
   const auto xstrlen = gridInqXIsc(gridID);
   const auto ystrlen = gridInqYIsc(gridID);
+  int datatype = CDI_UNDEFID;
+  cdiInqKeyInt(gridID, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
 
   char **xcvals = nullptr;
   if (xstrlen)
@@ -478,9 +469,8 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
   cdiInqKeyBytes(gridID, CDI_GLOBAL, CDI_KEY_UUID, uuid, &length);
   if (!cdiUUIDIsNull(uuid))
     {
-      char uuidStr[37];
-      cdiUUID2Str(uuid, uuidStr);
-      if (uuidStr[0] != 0 && strlen(uuidStr) == 36) fprintf(fp, "uuid      = %s\n", uuidStr);
+      char uuidStr[uuidNumHexChars + 1] = { 0 };
+      if (cdiUUID2Str(uuid, uuidStr) == uuidNumHexChars) fprintf(fp, "uuid      = %s\n", uuidStr);
     }
 
   if (gridInqMask(gridID, nullptr))
