@@ -1,27 +1,8 @@
-/*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
-
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-*/
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
+#include <utility>
 
 #include "cdo_output.h"
 #include "remap.h"
@@ -30,7 +11,7 @@
 /*****************************************************************************/
 
 static int
-isSorted(size_t *restrict array1, size_t *restrict array2, const size_t size)
+isSorted(size_t *array1, size_t *array2, const size_t size)
 {
   for (size_t idx = 1; idx < size; ++idx)
     {
@@ -42,75 +23,9 @@ isSorted(size_t *restrict array1, size_t *restrict array2, const size_t size)
 
   return 1;
 }
-/*
-static
-void swap(int *restrict v1, int *restrict v2)
-{
-  const int tmp = *v1;
-  *v1 = *v2;
-  *v2 = tmp;
-}
-
-static
-void heapify(const long pos, const long size, int *restrict arr1, int *restrict
-arr2, int *restrict arr3)
-{
-  const long leftChild = pos*2 + 1;
-
-  if ( leftChild < size )
-    {
-      long maxIter = pos;
-      if (  (arr1[leftChild] >  arr1[maxIter]) ||
-           ((arr1[leftChild] == arr1[maxIter]) &&
-            (arr2[leftChild] >  arr2[maxIter])) )
-        {
-          maxIter = leftChild;
-        }
-
-      const long rightChild = leftChild + 1;
-      if ( rightChild < size )
-        if (  (arr1[rightChild] >  arr1[maxIter]) ||
-             ((arr1[rightChild] == arr1[maxIter]) &&
-              (arr2[rightChild] >  arr2[maxIter])) )
-          {
-            maxIter = rightChild;
-          }
-
-      if ( maxIter != pos )
-        {
-          swap(&arr1[maxIter], &arr1[pos]);
-          swap(&arr2[maxIter], &arr2[pos]);
-          swap(&arr3[maxIter], &arr3[pos]);
-          heapify(maxIter, size, arr1, arr2, arr3);
-        }
-    }
-}
-*/
-/* remap_heapsort is 30% faster than remap_heapsort_recursiv ! */
-/*
-static
-void remap_heapsort_recursiv(const long num_links, int *restrict arr1, int
-*restrict arr2, int *restrict arr3)
-{
-  long lvl;     // level indexes for heap sort levels
-
-  for ( lvl = num_links/2-1; lvl >= 0; --lvl )
-    {
-      heapify(lvl, num_links, arr1, arr2, arr3);
-    }
-
-  for ( lvl = num_links-1; lvl > 0; --lvl )
-    {
-      swap(&arr1[0], &arr1[lvl]);
-      swap(&arr2[0], &arr2[lvl]);
-      swap(&arr3[0], &arr3[lvl]);
-      heapify(0, lvl, arr1, arr2, arr3);
-    }
-}
-*/
 
 static void
-remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict add2, size_t *restrict idx)
+remap_heapsort(const size_t num_links, size_t *add1, size_t *add2, size_t *idx)
 {
   size_t add1_tmp, add2_tmp; /* temp for addresses during swap     */
   size_t idx_tmp;
@@ -175,7 +90,7 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
             }
         }
 
-      if (i == num_links) cdoAbort("Internal problem, link 1 not found!");
+      if (i == num_links) cdo_abort("Internal problem, link 1 not found!");
     }
 
   // Now that the heap has been sorted, strip off the top (largest) value and promote the values below
@@ -239,7 +154,7 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
             }
         }
 
-      if (i == num_links) cdoAbort("Internal problem, link 2 not found!");
+      if (i == num_links) cdo_abort("Internal problem, link 2 not found!");
     }
 
   // Swap the last two entries
@@ -249,7 +164,7 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
 }
 
 void
-sort_add(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restrict add2, double *restrict weights)
+sort_add(size_t num_links, size_t num_wts, size_t *add1, size_t *add2, double *weights)
 {
   /*
     This routine sorts address and weight arrays based on the destination
@@ -259,9 +174,9 @@ sort_add(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restri
   /*
     Input and Output arrays:
 
-       long num_links; ! num of links for this mapping
-       long num_wts;   ! num of weights for this mapping
-       add1,           ! destination address array [num_links]
+       long num_links  ! num of links for this mapping
+       long num_wts    ! num of weights for this mapping
+       add1            ! destination address array [num_links]
        add2            ! source      address array
        weights         ! remapping weights [num_links*num_wts]
   */
@@ -355,8 +270,7 @@ merge_lists(size_t *nl, size_t *l11, size_t *l12, size_t *l21, size_t *l22, size
 }
 
 static void
-sort_par(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restrict add2, double *restrict weights, int parent,
-         int par_depth)
+sort_par(size_t num_links, size_t num_wts, size_t *add1, size_t *add2, double *weights, int parent, int par_depth)
 {
   /*
     This routine is the core of merge-sort. It does the following
@@ -520,7 +434,7 @@ sort_par(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restri
 }
 
 void
-sort_iter(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restrict add2, double *restrict weights, int parent)
+sort_iter(size_t num_links, size_t num_wts, size_t *add1, size_t *add2, double *weights, int parent)
 {
   /*
     This routine is an interface between the parallelized (merge-sort)
@@ -556,8 +470,7 @@ sort_iter(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restr
       parent = 1;
     }
 
-  // fprintf(stdout, "parent %d par_depth %d num_links %zu\n", parent,
-  // par_depth, num_links);
+  // fprintf(stdout, "parent %d par_depth %d num_links %zu\n", parent, par_depth, num_links);
 
   if (num_links > MERGE_SORT_LIMIT_SIZE && parent <= (nthreads - 1))
     {
@@ -566,8 +479,7 @@ sort_iter(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restr
     }
   else
     {
-      // printf("sort_add: parent %d, par_depth %d num_links %tz\n", parent,
-      // par_depth, num_links);
+      // printf("sort_add: parent %d, par_depth %d num_links %tz\n", parent, par_depth, num_links);
       sort_add(num_links, num_wts, add1, add2, weights);
     }
 }
